@@ -19,7 +19,7 @@
     Pete Shinners
     pete@shinners.org
 */
-
+#define NO_PYGAME_C_API
 #define PYGAMEAPI_BASE_INTERNAL
 #include "pygame.h"
 #include <signal.h>
@@ -45,7 +45,7 @@ QDGlobals qd;
 #endif
 
 #if defined(darwin)
-extern void StartTheDamnApplication(void);
+extern void StartTheApplication(void);
 extern void WeAreDoneFreeSomeMemory(void);
 #endif
 
@@ -160,7 +160,6 @@ static PyObject* register_quit(PyObject* self, PyObject* arg)
 
 static PyObject* init(PyObject* self,PyObject* args)
 {
-	static int initialized_once = 0;
 	PyObject *allmodules, *moduleslist, *dict, *func, *result, *mod;
 	int loop, num;
 	int success=0, fail=0;
@@ -171,26 +170,8 @@ static PyObject* init(PyObject* self,PyObject* args)
 		return NULL;
 
 
-/* let SDL do some basic initialization */
-	if(!initialized_once)
-	{
-		initialized_once = 1;
-#ifdef MS_WIN32
-		SDL_RegisterApp("pygame", 0, GetModuleHandle(NULL));
-#endif
-#if defined(macintosh)
-#if(!defined(__MWERKS__) && !TARGET_API_MAC_CARBON)
-		SDL_InitQuickDraw(&qd);
-#endif
-#endif
-#if defined(darwin)
-                StartTheDamnApplication();
-#endif
-		/*nice to initialize timer, so startup time will be correct before init call*/
-		SDL_Init(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
-		Py_AtExit(atexit_quit);
-		install_parachute();
-	}
+	/*nice to initialize timer, so startup time will reflec init() time*/
+	SDL_Init(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
 
 
 /* initialize all pygame modules */
@@ -244,8 +225,8 @@ static void atexit_quit(void)
 	quitfunctions = NULL;
 
 	uninstall_parachute();
-
 	num = PyList_Size(privatefuncs);
+
 	while(num--) /*quit in reverse order*/
 	{
 		quit = PyList_GET_ITEM(privatefuncs, num);
@@ -258,10 +239,9 @@ static void atexit_quit(void)
 		}
 	}
 	Py_DECREF(privatefuncs);
-	SDL_QuitSubSystem(SDL_INIT_TIMER);
-	SDL_Quit(); /*catch anything left*/
+	SDL_Quit();
 #if defined(darwin)
-        WeAreDoneFreeSomeMemory();
+	WeAreDoneFreeSomeMemory();
 #endif
 }
 
@@ -338,6 +318,50 @@ static int TwoShortsFromObj(PyObject* obj, short* val1, short* val2)
 	return 1;
 }
 
+
+static int FloatFromObj(PyObject* obj, float* val)
+{
+	PyObject* floatobj;
+
+	if(PyNumber_Check(obj))
+	{
+		if(!(floatobj = PyNumber_Float(obj)))
+			return 0;
+		*val = (float)PyFloat_AsDouble(floatobj);
+		Py_DECREF(floatobj);
+		return 1;
+	}
+	return 0;
+}
+
+static int FloatFromObjIndex(PyObject* obj, int index, float* val)
+{
+	int result = 0;
+	PyObject* item;
+	item = PySequence_GetItem(obj, index);
+	if(item)
+	{
+		result = FloatFromObj(item, val);
+		Py_DECREF(item);
+	}
+	return result;
+}
+
+static int TwoFloatsFromObj(PyObject* obj, float* val1, float* val2)
+{
+	if(PyTuple_Check(obj) && PyTuple_Size(obj)==1)
+		return TwoFloatsFromObj(PyTuple_GET_ITEM(obj, 0), val1, val2);
+
+	if(!PySequence_Check(obj) || PySequence_Length(obj) != 2)
+		return 0;
+
+	if(!FloatFromObjIndex(obj, 0, val1) || !FloatFromObjIndex(obj, 1, val2))
+		return 0;
+
+	return 1;
+}
+
+
 static int UintFromObj(PyObject* obj, Uint32* val)
 {
 	PyObject* intobj;
@@ -501,6 +525,7 @@ static void pygame_parachute(int sig)
 #ifdef DO_CRASH_TRACEBACK
 	PyThreadState* thread;
 	PyInterpreterState *interp;
+	int thread_id;
 #endif	  
     
 	signal(sig, SIG_DFL);
@@ -530,17 +555,28 @@ static void pygame_parachute(int sig)
 			signaltype = "(pygame parachute) Unknown Signal"; break;
 	}
 
+	
 #ifdef DO_CRASH_TRACEBACK
-printf("DOING TRACEBACK, experimental...\n");
+	printf("Pygame Parachute Traceback:\n");
 	interp = PyInterpreterState_Head();
-	for(thread=PyInterpreterState_ThreadHead(interp); thread; thread = PyThreadState_Next(thread))
+	thread=PyInterpreterState_ThreadHead(interp);
+	if(PyThreadState_Next(thread)) /*multithreaded*/
+	    thread_id = 0;
+	else
+	    thread_id = -1; /*no threads, don't print thread info*/
+	for(; thread; thread = PyThreadState_Next(thread))
 	{
+	    if(thread_id >= 0)
+	    {
+		printf("Thread-%p\n", thread);
+		thread_id++;
+	    }
 	    PyTraceBack_Here(thread->frame);
 	    Py_INCREF(thread->curexc_traceback);
 	    print_traceback(thread->curexc_traceback);
 	}
 #else
-printf("  (No Traceback Without Python2.2)\n");
+	printf("  (No Traceback Without Python2.2)\n");
 #endif
 	
 	atexit_quit();
@@ -568,12 +604,16 @@ static int fatal_signals[] =
 	0 /*end of list*/
 };
 
+static int parachute_installed = 0;
 static void install_parachute(void)
 {
 	int i;
 	void (*ohandler)(int);
 
-#if 1
+	if(parachute_installed)
+	    return;
+	parachute_installed = 1;
+
 	/* Set a handler for any fatal signal not already handled */
 	for ( i=0; fatal_signals[i]; ++i )
 	{
@@ -592,7 +632,6 @@ static void install_parachute(void)
 		if ( oaction.sa_handler != SIG_DFL )
 			sigaction(SIGALRM, &oaction, NULL);
 	}
-#endif
 #endif
 	return;
 }
@@ -603,6 +642,10 @@ static void uninstall_parachute(void)
 	int i;
 	void (*ohandler)(int);
 
+	if(!parachute_installed)
+	    return;
+	parachute_installed = 0;
+
 	/* Remove a handler for any fatal signal handled */
 	for ( i=0; fatal_signals[i]; ++i ) {
 		ohandler = signal(fatal_signals[i], SIG_DFL);
@@ -674,18 +717,32 @@ void initbase(void)
 	c_api[2] = ShortFromObj;
 	c_api[3] = ShortFromObjIndex;
 	c_api[4] = TwoShortsFromObj;
-	c_api[5] = UintFromObj;
-	c_api[6] = UintFromObjIndex;
-	c_api[7] = PyGame_Video_AutoQuit;
-	c_api[8] = PyGame_Video_AutoInit;
-	c_api[9] = RGBAFromObj;
+	c_api[5] = FloatFromObj;
+	c_api[6] = FloatFromObjIndex;
+	c_api[7] = TwoFloatsFromObj;
+	c_api[8] = UintFromObj;
+	c_api[9] = UintFromObjIndex;
+	c_api[10] = PyGame_Video_AutoQuit;
+	c_api[11] = PyGame_Video_AutoInit;
+	c_api[12] = RGBAFromObj;
 	apiobj = PyCObject_FromVoidPtr(c_api, NULL);
 	PyDict_SetItemString(dict, PYGAMEAPI_LOCAL_ENTRY, apiobj);
 	Py_DECREF(apiobj);
 
-
-	/*touch PyGAME_C_API to keep compiler from warning*/
-	PyGAME_C_API[0] = PyGAME_C_API[0];
+	/*some intiialization*/
+	Py_AtExit(atexit_quit);
+	install_parachute();
+#ifdef MS_WIN32
+	SDL_RegisterApp("pygame", 0, GetModuleHandle(NULL));
+#endif
+#if defined(macintosh)
+#if(!defined(__MWERKS__) && !TARGET_API_MAC_CARBON)
+	SDL_InitQuickDraw(&qd);
+#endif
+#endif
+#if defined(darwin)
+	StartTheApplication();
+#endif
 }
 
 
