@@ -19,7 +19,7 @@
     Pete Shinners
     pete@shinners.org
 */
-
+#define NO_PYGAME_C_API
 #include "pygame.h"
 
 
@@ -51,9 +51,6 @@ void initconstants(void)
 {
 	PyObject* module;
 
-	PyGAME_C_API[0] = PyGAME_C_API[0]; /*this cleans up compiler warning*/
-
-
 	module = Py_InitModule3("constants", builtins, doc_pygame_constants_MODULE);
 
 	DEC_CONST(SWSURFACE);
@@ -314,6 +311,7 @@ void initconstants(void)
     /*DOC*/    "FULLSCREEN - display surface is fullscreen (nonwindowed)<br>\n"
     /*DOC*/    "RLEACCEL - compile for quick alpha blits, only set in alpha or\n"
     /*DOC*/    "colorkey funcs<br>\n"
+    /*DOC*/    "NOFRAME - no window decorations<br>\n"
     /*DOC*/ ;
 
     /*DOC*/ static char doc_events[] =
