@@ -26,7 +26,6 @@
 #define PYGAMEAPI_DISPLAY_INTERNAL
 #include "pygame.h"
 
-
 static char* icon_defaultname = "pygame_icon.bmp";
 static PyObject* self_module = NULL;
 
@@ -34,6 +33,8 @@ static PyObject* self_module = NULL;
 staticforward PyTypeObject PyVidInfo_Type;
 static PyObject* PyVidInfo_New(const SDL_VideoInfo* info);
 static PyObject* DisplaySurfaceObject = NULL;
+static int icon_was_set = 0;
+
 
 #if 0
 /*quick internal test to see if gamma is supported*/
@@ -57,9 +58,9 @@ static void display_autoquit(void)
 {
         if(DisplaySurfaceObject)
         {
-                /*Ack, this still leaves pretty deadly Surface objs around*/
 		((PySurfaceObject*)DisplaySurfaceObject)->surf = NULL;
                 Py_DECREF(DisplaySurfaceObject);
+                DisplaySurfaceObject = NULL;
         }
 }
 
@@ -375,7 +376,73 @@ static PyObject* get_surface(PyObject* self, PyObject* arg)
 	return DisplaySurfaceObject;
 }
 
+    /*DOC*/ static char doc_gl_set_attribute[] =
+    /*DOC*/    "pygame.display.gl_set_attribute(flag, value) -> None\n"
+    /*DOC*/    "set special OPENGL attributes\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "When calling pygame.display.set_mode() with the OPENGL flag,\n"
+    /*DOC*/    "pygame automatically handles setting the opengl attributes like\n"
+    /*DOC*/    "color and doublebuffering. OPENGL offers several other attributes\n"
+    /*DOC*/    "you may want control over. Pass one of these attributes as the\n"
+    /*DOC*/    "flag, and its appropriate value.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "This must be called before pygame.display.set_mode()\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "The OPENGL flags are; GL_ALPHA_SIZE, GL_DEPTH_SIZE, GL_STENCIL_SIZE,\n"
+    /*DOC*/    "GL_ACCUM_RED_SIZE, GL_ACCUM_GREEN_SIZE, GL_ACCUM_BLUE_SIZE, GL_ACCUM_ALPHA_SIZE\n"
+    /*DOC*/ ;
 
+static PyObject* gl_set_attribute(PyObject* self, PyObject* arg)
+{
+        int flag, value, result;
+    
+	VIDEO_INIT_CHECK();
+
+    	if(!PyArg_ParseTuple(arg, "ii", &flag, &value))
+
+		return NULL;
+	result = SDL_GL_SetAttribute(flag, value);
+        if(result == -1)
+            return RAISE(PyExc_SDLError, SDL_GetError());
+        
+        RETURN_NONE
+}
+
+
+    /*DOC*/ static char doc_gl_get_attribute[] =
+    /*DOC*/    "pygame.display.gl_get_attribute(flag) -> value\n"
+    /*DOC*/    "get special OPENGL attributes\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "After calling pygame.display.set_mode() with the OPENGL flag\n"
+    /*DOC*/    "you will likely want to check the value of any special opengl\n"
+    /*DOC*/    "attributes you requested. You will not always get what you\n"
+    /*DOC*/    "requested.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "See pygame.display.gl_set_attribute() for a list of flags.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "The OPENGL flags are; GL_ALPHA_SIZE, GL_DEPTH_SIZE, GL_STENCIL_SIZE,\n"
+    /*DOC*/    "GL_ACCUM_RED_SIZE, GL_ACCUM_GREEN_SIZE, GL_ACCUM_BLUE_SIZE, GL_ACCUM_ALPHA_SIZE,\n"
+    /*DOC*/    "GL_RED_SIZE, GL_GREEN_SIZE, GL_BLUE_SIZE, GL_DEPTH_SIZE\n"
+    /*DOC*/ ;
+
+static PyObject* gl_get_attribute(PyObject* self, PyObject* arg)
+{
+        int flag, value, result;
+    
+	VIDEO_INIT_CHECK();
+
+	if(!PyArg_ParseTuple(arg, "i", &flag))
+		return NULL;
+
+	result = SDL_GL_GetAttribute(flag, &value);
+        if(result == -1)
+            return RAISE(PyExc_SDLError, SDL_GetError());
+        
+        return PyInt_FromLong(value);
+}
+
+
+            
     /*DOC*/ static char doc_set_mode[] =
     /*DOC*/    "pygame.display.set_mode(size, [flags, [depth]]) -> Surface\n"
     /*DOC*/    "set the display mode\n"
@@ -393,12 +460,14 @@ static PyObject* get_surface(PyObject* self, PyObject* arg)
     /*DOC*/    "flags together with the bitwise-or (|) operator. Possible flags\n"
     /*DOC*/    "are HWSURFACE (or the value 1), HWPALETTE, DOUBLEBUF, and/or\n"
     /*DOC*/    "FULLSCREEN. There are other flags available but these are the\n"
-    /*DOC*/    "most usual. A full list of flags can be found in the SDL\n"
-    /*DOC*/    "documentation. The optional depth arguement is the requested bits\n"
+    /*DOC*/    "most usual. A full list of flags can be found in the pygame\n"
+    /*DOC*/    "documentation.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "The optional depth arguement is the requested bits\n"
     /*DOC*/    "per pixel. It will usually be left omitted, in which case the\n"
     /*DOC*/    "display will use the best/fastest pixel depth available.\n"
     /*DOC*/    "\n"
-    /*DOC*/    "You can also create an OpenGL surface (for use with PyOpenGL)\n"
+    /*DOC*/    "You can create an OpenGL surface (for use with PyOpenGL)\n"
     /*DOC*/    "by passing the OPENGL flag. You will likely want to use the\n"
     /*DOC*/    "DOUBLEBUF flag when using OPENGL. In which case, the flip()\n"
     /*DOC*/    "function will perform the GL buffer swaps. When you are using\n"
@@ -411,8 +480,7 @@ static PyObject* set_mode(PyObject* self, PyObject* arg)
 	SDL_Surface* surf;
 	int flags = SDL_SWSURFACE, depth = 0;
 	int w, h, hasbuf;
-	char* title, *icontitle;
-	static int icon_was_set = 0;
+	char *title, *icontitle;
 
 	if(!PyArg_ParseTuple(arg, "(ii)|ii", &w, &h, &flags, &depth))
 		return NULL;
@@ -435,9 +503,7 @@ static PyObject* set_mode(PyObject* self, PyObject* arg)
 			SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 0);
 		if(depth)
 			SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, depth);
-		Py_BEGIN_ALLOW_THREADS
 		surf = SDL_SetVideoMode(w, h, depth, flags);
-		Py_END_ALLOW_THREADS
 		if(!surf)
 			return RAISE(PyExc_SDLError, SDL_GetError());
 
@@ -457,12 +523,10 @@ static PyObject* set_mode(PyObject* self, PyObject* arg)
 		if(!surf)
 			return RAISE(PyExc_SDLError, SDL_GetError());
 	}
-
 	SDL_WM_GetCaption(&title, &icontitle);
 	if(!title || !*title)
 		SDL_WM_SetCaption("pygame window", "pygame");
 
-
 	/*probably won't do much, but can't hurt, and might help*/
 	SDL_PumpEvents();
 
@@ -506,7 +570,6 @@ static PyObject* set_mode(PyObject* self, PyObject* arg)
 		}
 	}
 #endif
-
 	Py_INCREF(DisplaySurfaceObject);
 	return DisplaySurfaceObject;
 }
@@ -558,7 +621,7 @@ static PyObject* mode_ok(PyObject* self, PyObject* args)
     /*DOC*/    "query all resolutions for requested mode\n"
     /*DOC*/    "\n"
     /*DOC*/    "This function returns a list of possible dimensions for a\n"
-    /*DOC*/    "specified color depth. The return value will be an empty list of\n"
+    /*DOC*/    "specified color depth. The return value will be an empty list if\n"
     /*DOC*/    "no display modes are available with the given arguments. A return\n"
     /*DOC*/    "value of -1 means that any requested resolution should work (this\n"
     /*DOC*/    "is likely the case for windowed modes). Mode sizes are sorted\n"
@@ -656,9 +719,8 @@ static PyObject* flip(PyObject* self, PyObject* arg)
 static GAME_Rect* screencroprect(GAME_Rect* r, int w, int h, GAME_Rect* cur)
 {
 	if(r->x > w || r->y > h || (r->x + r->w) <= 0 || (r->y + r->h) <= 0)
-{printf("update rejecting rect %d,%d,%d,%d\n", (int)r->x, (int)r->y, (int)r->w, (int)r->h);
                 return 0;
-}       else
+        else
 	{
 		int right = min(r->x + r->w, w);
 		int bottom = min(r->y + r->h, h);
@@ -702,7 +764,6 @@ static PyObject* update(PyObject* self, PyObject* arg)
 	high = screen->h;
 	if(screen->flags & SDL_OPENGL)
 		return RAISE(PyExc_SDLError, "Cannot update an OPENGL display");
-
         
 	/*determine type of argument we got*/
 	if(PyTuple_Size(arg) == 0)
@@ -780,7 +841,8 @@ static PyObject* update(PyObject* self, PyObject* arg)
                         ++count;
                 }
         
-                SDL_UpdateRects(screen, count, rects);
+                if(count)
+                    SDL_UpdateRects(screen, count, rects);
                 PyMem_Free((char*)rects);
 	}
 	RETURN_NONE
@@ -896,7 +958,93 @@ static PyObject* set_gamma(PyObject* self, PyObject* arg)
 	return PyInt_FromLong(result == 0);
 }
 
+static int convert_to_uint16(PyObject* python_array, Uint16* c_uint16_array)
+{
+        int i;
+        PyObject* item;
 
+	if (!c_uint16_array) {
+	        RAISE(PyExc_RuntimeError, "Memory not allocated for c_uint16_array.");
+		return 0;
+	}
+
+	if (!PySequence_Check(python_array))
+        {
+	        RAISE(PyExc_TypeError, "Array must be sequence type");
+	        return 0;
+        }
+
+	if (PySequence_Size(python_array) != 256)
+        {
+		RAISE(PyExc_ValueError, "gamma ramp must be 256 elements long");
+                return 0;
+        }
+	for (i=0; i<256; i++)
+        {
+                item = PySequence_GetItem(python_array, i);
+                if(!PyInt_Check(item))
+                {
+		    RAISE(PyExc_ValueError, "gamma ramp must contain integer elements");
+                    return 0;
+                }
+		c_uint16_array[i] = (Uint16)PyInt_AsLong(item);
+        }
+	return 1;
+}
+
+    /*DOC*/ static char doc_set_gamma_ramp[] =
+    /*DOC*/    "pygame.display.set_gamma_ramp(r, g, b) -> bool\n"
+    /*DOC*/    "advanced control over the display gamma ramps\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Pass three sequences with 256 elements. Each element must be a\n"
+    /*DOC*/    "'16bit' unsigned integer value. This is from 0 to 65536.\n"
+    /*DOC*/    "If you are using a display mode with a hardware\n"
+    /*DOC*/    "palette, this will simply update the palette you are using.\n"
+    /*DOC*/    "Not all hardware supports gamma. The return value will be\n"
+    /*DOC*/    "true on success.\n";
+
+static PyObject* set_gamma_ramp(PyObject* self, PyObject* arg)
+{
+ 	Uint16 *r, *g, *b;
+
+	int result;
+ 
+	r = (Uint16 *)malloc(256 * sizeof(Uint16));
+	if (!r)
+	        return NULL;
+	g = (Uint16 *)malloc(256 * sizeof(Uint16));
+	if (!g)
+        {
+                free(r);
+	        return NULL;
+        }
+	b = (Uint16 *)malloc(256 * sizeof(Uint16));
+	if (!b)
+        {
+                free(r);
+                free(g);
+	        return NULL;
+        }
+
+	if(!PyArg_ParseTuple(arg, "O&O&O&", 
+			     convert_to_uint16, r,
+			     convert_to_uint16, g,
+			     convert_to_uint16, b))
+        {
+                free(r); free(g); free(b);
+	        return NULL;
+        }
+
+        VIDEO_INIT_CHECK();
+	
+	result = SDL_SetGammaRamp(r, g, b);
+
+	free((char*)r);
+	free((char*)g);
+	free((char*)b);
+
+	return PyInt_FromLong(result == 0);
+}
 
     /*DOC*/ static char doc_set_caption[] =
     /*DOC*/    "pygame.display.set_caption(title, [icontitle]) -> None\n"
@@ -954,12 +1102,14 @@ static PyObject* get_caption(PyObject* self, PyObject* arg)
     /*DOC*/    "\n"
     /*DOC*/    "Sets the runtime icon that your system uses to decorate\n"
     /*DOC*/    "the program window. It is also used when the application\n"
-    /*DOC*/    "is iconfified. This must be called before the first call\n"
-    /*DOC*/    "to pygame.display.set_mode().\n"
+    /*DOC*/    "is iconified and in the window frame.\n"
     /*DOC*/    "\n"
     /*DOC*/    "You likely want this to be a smaller image, a size that\n"
     /*DOC*/    "your system window manager will be able to deal with. It will\n"
     /*DOC*/    "also use the Surface colorkey if available.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Some window managers on X11 don't allow you to change the \n"
+    /*DOC*/    "icon after the window has been shown the first time.\n"
     /*DOC*/ ;
 
 static PyObject* set_icon(PyObject* self, PyObject* arg)
@@ -974,6 +1124,7 @@ static PyObject* set_icon(PyObject* self, PyObject* arg)
 	SDL_WM_SetIcon(surf, NULL);
 	PySurface_Unlock(surface);
 
+	icon_was_set = 1;
 	RETURN_NONE
 }
 
@@ -1049,21 +1200,22 @@ static PyMethodDef display_builtins[] =
 	{ "mode_ok", mode_ok, 1, doc_mode_ok },
 	{ "list_modes", list_modes, 1, doc_list_modes },
 
-	{ "flip", flip, 1, doc_flip },
-	{ "update", update, 1, doc_update },
+        { "flip", flip, 1, doc_flip }, { "update", update, 1, doc_update },
 
 	{ "set_palette", set_palette, 1, doc_set_palette },
 	{ "set_gamma", set_gamma, 1, doc_set_gamma },
-	/*gammaramp support will be added later, if needed?*/
+	{ "set_gamma_ramp", set_gamma_ramp, 1, doc_set_gamma_ramp },
 
 	{ "set_caption", set_caption, 1, doc_set_caption },
 	{ "get_caption", get_caption, 1, doc_get_caption },
 	{ "set_icon", set_icon, 1, doc_set_icon },
 
-	/*{ "set_icon", set_icon, 1, doc_set_icon }, need to wait for surface objects*/
 	{ "iconify", iconify, 1, doc_iconify },
 	{ "toggle_fullscreen", toggle_fullscreen, 1, doc_toggle_fullscreen },
 
+	{ "gl_set_attribute", gl_set_attribute, 1, doc_gl_set_attribute },
+	{ "gl_get_attribute", gl_get_attribute, 1, doc_gl_get_attribute },
+
 	{ NULL, NULL }
 };
 
