@@ -152,6 +152,58 @@ static void insobj(PyObject *dict, char *name, PyObject *v)
 	}
 }
 
+#ifdef Py_USING_UNICODE
+
+static PyObject* our_unichr(long uni)
+{
+	static PyObject* bltin_unichr = NULL;
+
+	if (bltin_unichr == NULL) {
+		PyObject* bltins;
+		
+		bltins = PyImport_ImportModule("__builtin__");
+		bltin_unichr = PyObject_GetAttrString(bltins, "unichr");
+
+		Py_DECREF(bltins);
+	}
+
+	return PyEval_CallFunction(bltin_unichr, "(l)", uni);
+}
+
+static PyObject* our_empty_ustr(void)
+{
+	static PyObject* empty_ustr = NULL;
+
+	if (empty_ustr == NULL) {
+		PyObject* bltins;
+		PyObject* bltin_unicode;
+		
+		bltins = PyImport_ImportModule("__builtin__");
+		bltin_unicode = PyObject_GetAttrString(bltins, "unicode");
+		empty_ustr = PyEval_CallFunction(bltin_unicode, "(s)", "");
+
+		Py_DECREF(bltin_unicode);
+		Py_DECREF(bltins);
+	}
+
+	Py_INCREF(empty_ustr);
+
+	return empty_ustr;
+}
+
+#else
+
+static PyObject* our_unichr(long uni)
+{
+	return PyInt_FromLong(uni);
+}
+
+static PyObject* our_empty_ustr(void)
+{
+	return PyInt_FromLong(0);
+}
+
+#endif /* Py_USING_UNICODE */
 
 static PyObject* dict_from_event(SDL_Event* event)
 {
@@ -166,7 +218,6 @@ static PyObject* dict_from_event(SDL_Event* event)
 			return dict;
 	}
 
-
 	if(!(dict = PyDict_New()))
 		return NULL;
 	switch(event->type)
@@ -177,12 +228,9 @@ static PyObject* dict_from_event(SDL_Event* event)
 		break;
 	case SDL_KEYDOWN:
 		if(event->key.keysym.unicode)
-			insobj(dict, "unicode", PyUnicode_FromUnicode(
-							(Py_UNICODE*)&event->key.keysym.unicode,
-							event->key.keysym.unicode > 0));
+			insobj(dict, "unicode", our_unichr(event->key.keysym.unicode));
 		else
-			insobj(dict, "unicode",
-							PyUnicode_FromObject(PyString_FromString("")));
+			insobj(dict, "unicode", our_empty_ustr());
 	case SDL_KEYUP:
 		insobj(dict, "key", PyInt_FromLong(event->key.keysym.sym));
 		insobj(dict, "mod", PyInt_FromLong(event->key.keysym.mod));
@@ -313,7 +361,7 @@ PyObject* event_str(PyObject* self)
     /*DOC*/    "lookups will be passed through to the Event's dictionary values.\n"
     /*DOC*/    "\n"
     /*DOC*/    "While debugging and experimenting, you can print the Event\n"
-    /*DOC*/    "objects for a quick display of its type and members.\n"    __SECRET_COLON__ 
+    /*DOC*/    "objects for a quick display of its type and members.\n" __SECRET_COLON__ 
     /*DOC*/    "Events that come from the system will have a guaranteed set of\n"
     /*DOC*/    "member items based on the type. Here is a list of the Event members\n"
     /*DOC*/    "that are defined with each type.<br><table align=center>"
@@ -334,7 +382,36 @@ PyObject* event_str(PyObject* self)
     /*DOC*/    "<tr><td><b>USEREVENT</b></td><td>code</td></tr></table>\n"
     /*DOC*/ ;
 
+static int event_nonzero(PyEventObject *self)
+{
+	return self->type != SDL_NOEVENT;
+}
 
+static PyNumberMethods event_as_number = {
+	(binaryfunc)NULL,		/*add*/
+	(binaryfunc)NULL,		/*subtract*/
+	(binaryfunc)NULL,		/*multiply*/
+	(binaryfunc)NULL,		/*divide*/
+	(binaryfunc)NULL,		/*remainder*/
+	(binaryfunc)NULL,		/*divmod*/
+	(ternaryfunc)NULL,		/*power*/
+	(unaryfunc)NULL,		/*negative*/
+	(unaryfunc)NULL,		/*pos*/ 
+	(unaryfunc)NULL,		/*abs*/
+	(inquiry)event_nonzero,	/*nonzero*/
+	(unaryfunc)NULL,		/*invert*/
+	(binaryfunc)NULL,		/*lshift*/
+	(binaryfunc)NULL,		/*rshift*/
+	(binaryfunc)NULL,		/*and*/
+	(binaryfunc)NULL,		/*xor*/
+	(binaryfunc)NULL,		/*or*/
+	(coercion)NULL,			/*coerce*/
+	(unaryfunc)NULL,		/*int*/
+	(unaryfunc)NULL,		/*long*/
+	(unaryfunc)NULL,		/*float*/
+	(unaryfunc)NULL,		/*oct*/
+	(unaryfunc)NULL,		/*hex*/
+};
 
 
 static PyTypeObject PyEvent_Type =
@@ -350,7 +427,7 @@ static PyTypeObject PyEvent_Type =
 	NULL,					/*setattr*/
 	NULL,					/*compare*/
 	event_str,				/*repr*/
-	NULL,					/*as_number*/
+	&event_as_number,		/*as_number*/
 	NULL,					/*as_sequence*/
 	NULL,					/*as_mapping*/
 	(hashfunc)NULL, 		/*hash*/
@@ -366,12 +443,18 @@ static PyObject* PyEvent_New(SDL_Event* event)
 {
 	PyEventObject* e;
 	e = PyObject_NEW(PyEventObject, &PyEvent_Type);
+	if(!e) return NULL;
 
-	if(e)
+	if(event)
 	{
 		e->type = event->type;
 		e->dict = dict_from_event(event);
 	}
+	else
+	{
+		e->type = SDL_NOEVENT;
+		e->dict = PyDict_New();
+	}
 	return (PyObject*)e;
 }
 
@@ -397,7 +480,7 @@ static PyObject* PyEvent_New2(int type, PyObject* dict)
 
 
     /*DOC*/ static char doc_Event[] =
-    /*DOC*/    "pygame.event.Event(type, dict, [keyword_args]) -> Event\n"
+    /*DOC*/    "pygame.event.Event(type, [dict], [keyword_args]) -> Event\n"
     /*DOC*/    "create new event object\n"
     /*DOC*/    "\n"
     /*DOC*/    "Creates a new event object. The type should be one of SDL's\n"
@@ -405,8 +488,8 @@ static PyObject* PyEvent_New2(int type, PyObject* dict)
     /*DOC*/    "the keys that will be members of the new event.\n"
     /*DOC*/    "\n"
     /*DOC*/    "Also, instead of passing a dictionary to create the event\n"
-    /*DOC*/    "members, you also/or pass a list of keyword arguments that\n"
-    /*DOC*/    "will become data members in the new event.\n"
+    /*DOC*/    "members, you can pass keyword arguments that will become the\n"
+    /*DOC*/    "attributes of the new event.\n"
     /*DOC*/ ;
 
 static PyObject* Event(PyObject* self, PyObject* arg, PyObject* keywords)
@@ -514,11 +597,16 @@ static PyObject* get_grab(PyObject* self, PyObject* arg)
     /*DOC*/    "pygame.event.pump() -> None\n"
     /*DOC*/    "update the internal messages\n"
     /*DOC*/    "\n"
-    /*DOC*/    "Pumping the message queue is important if you are not getting\n"
-    /*DOC*/    "events off the message queue. The pump will allow pygame to\n"
-    /*DOC*/    "communicate with the window manager, which helps keep your\n"
-    /*DOC*/    "application responsive, as well as updating the state for various\n"
-    /*DOC*/    "input devices.\n"
+    /*DOC*/    "For each frame of your game, you will need to make some sort\n"
+    /*DOC*/    "of call to the event queue. This ensures your program can internally\n"
+    /*DOC*/    "interact with the rest of the operating system. If you are not using\n"
+    /*DOC*/    "other event functions in your game, you should call pump() to allow\n"
+    /*DOC*/    "pygame to handle internal actions.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "There are important things that must be dealt with internally in the\n"
+    /*DOC*/    "event queue. The main window may need to be repainted. Certain joysticks\n"
+    /*DOC*/    "must be polled for their values. If you fail to make a call to the event\n"
+    /*DOC*/    "queue for too long, the system may decide your program has locked up.\n"
     /*DOC*/ ;
 
 static PyObject* pump(PyObject* self, PyObject* args)
@@ -585,35 +673,34 @@ static PyObject* poll(PyObject* self, PyObject* args)
 
 	VIDEO_INIT_CHECK();
 
-	if(!SDL_PollEvent(&event))
-		return PyEvent_New2(SDL_NOEVENT, NULL);
-
-	return PyEvent_New(&event);
+	if(SDL_PollEvent(&event))
+		return PyEvent_New(&event);
+	return PyEvent_New(NULL);
 }
 
 
-    /*DOC*/ static char doc_get[] =
-    /*DOC*/    "pygame.event.get([type]) -> list of Events\n"
-    /*DOC*/    "get all of an event type from the queue\n"
+    /*DOC*/ static char doc_event_clear[] =
+    /*DOC*/    "pygame.event.clear([type]) -> None\n"
+    /*DOC*/    "remove all of an event type from the queue\n"
     /*DOC*/    "\n"
-    /*DOC*/    "Pass this a type of event that you are interested in, and it will\n"
-    /*DOC*/    "return a list of all matching event types from the queue. If no\n"
-    /*DOC*/    "types are passed, this will return all the events from the queue.\n"
+    /*DOC*/    "Pass this a type of event to discard, and it will\n"
+    /*DOC*/    "remove all matching event types from the queue. If no\n"
+    /*DOC*/    "types are passed, this will remove all the events from the queue.\n"
     /*DOC*/    "You may also optionally pass a sequence of event types. For\n"
-    /*DOC*/    "example, to fetch all the keyboard events from the queue, you\n"
-    /*DOC*/    "would call, 'pygame.event.get([KEYDOWN,KEYUP])'.\n"
+    /*DOC*/    "example, to remove all the mouse motion events from the queue, you\n"
+    /*DOC*/    "would call, 'pygame.event.clear(MOUSEMOTION)'.\n"
     /*DOC*/ ;
 
-static PyObject* get(PyObject* self, PyObject* args)
+static PyObject* event_clear(PyObject* self, PyObject* args)
 {
 	SDL_Event event;
 	int mask = 0;
 	int loop, num;
-	PyObject* type, *list, *e;
+	PyObject* type;
 	short val;
 
 	if(PyTuple_Size(args) != 0 && PyTuple_Size(args) != 1)
-		return RAISE(PyExc_ValueError, "peek requires 0 or 1 argument");
+		return RAISE(PyExc_ValueError, "get requires 0 or 1 argument");
 
 	VIDEO_INIT_CHECK();
 
@@ -635,7 +722,62 @@ static PyObject* get(PyObject* self, PyObject* args)
 		else if(ShortFromObj(type, &val))
 			mask = SDL_EVENTMASK(val);
 		else
-			return RAISE(PyExc_TypeError, "peek type must be numeric or a sequence");
+			return RAISE(PyExc_TypeError, "get type must be numeric or a sequence");
+	}
+
+	SDL_PumpEvents();
+
+	while(SDL_PeepEvents(&event, 1, SDL_GETEVENT, mask) == 1)
+	{}
+
+	RETURN_NONE;
+}
+
+
+    /*DOC*/ static char doc_event_get[] =
+    /*DOC*/    "pygame.event.get([type]) -> list of Events\n"
+    /*DOC*/    "get all of an event type from the queue\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Pass this a type of event that you are interested in, and it will\n"
+    /*DOC*/    "return a list of all matching event types from the queue. If no\n"
+    /*DOC*/    "types are passed, this will return all the events from the queue.\n"
+    /*DOC*/    "You may also optionally pass a sequence of event types. For\n"
+    /*DOC*/    "example, to fetch all the keyboard events from the queue, you\n"
+    /*DOC*/    "would call, 'pygame.event.get([KEYDOWN,KEYUP])'.\n"
+    /*DOC*/ ;
+
+static PyObject* event_get(PyObject* self, PyObject* args)
+{
+	SDL_Event event;
+	int mask = 0;
+	int loop, num;
+	PyObject* type, *list, *e;
+	short val;
+
+	if(PyTuple_Size(args) != 0 && PyTuple_Size(args) != 1)
+		return RAISE(PyExc_ValueError, "get requires 0 or 1 argument");
+
+	VIDEO_INIT_CHECK();
+
+	if(PyTuple_Size(args) == 0)
+		mask = SDL_ALLEVENTS;
+	else
+	{
+		type = PyTuple_GET_ITEM(args, 0);
+		if(PySequence_Check(type))
+		{
+			num = PySequence_Size(type);
+			for(loop=0; loop<num; ++loop)
+			{
+				if(!ShortFromObjIndex(type, loop, &val))
+					return RAISE(PyExc_TypeError, "type sequence must contain valid event types");
+				mask |= SDL_EVENTMASK(val);
+			}
+		}
+		else if(ShortFromObj(type, &val))
+			mask = SDL_EVENTMASK(val);
+		else
+			return RAISE(PyExc_TypeError, "get type must be numeric or a sequence");
 	}
 	
 	list = PyList_New(0);
@@ -667,8 +809,8 @@ static PyObject* get(PyObject* self, PyObject* args)
     /*DOC*/    "\n"
     /*DOC*/    "Pass this a type of event that you are interested in, and it will\n"
     /*DOC*/    "return true if there are any of that type of event on the queue.\n"
-    /*DOC*/    "If no types are passed, this will return true if any events are\n"
-    /*DOC*/    "on the queue. You may also optionally pass a sequence of event\n"
+    /*DOC*/    "If no types are passed, this will return the next event on the queue\n"
+    /*DOC*/    "without removing it. You may also optionally pass a sequence of event\n"
     /*DOC*/    "types. For example, to find if any keyboard events are on the\n"
     /*DOC*/    "queue, you would call, 'pygame.event.peek([KEYDOWN,KEYUP])'.\n"
     /*DOC*/ ;
@@ -678,7 +820,7 @@ static PyObject* event_peek(PyObject* self, PyObject* args)
 	SDL_Event event;
 	int result;
 	int mask = 0;
-	int loop, num;
+	int loop, num, noargs=0;
 	PyObject* type;
 	short val;
 
@@ -688,7 +830,10 @@ static PyObject* event_peek(PyObject* self, PyObject* args)
 	VIDEO_INIT_CHECK();
 
 	if(PyTuple_Size(args) == 0)
+	{
 		mask = SDL_ALLEVENTS;
+		noargs=1;
+	}
 	else
 	{
 		type = PyTuple_GET_ITEM(args, 0);
@@ -710,6 +855,9 @@ static PyObject* event_peek(PyObject* self, PyObject* args)
 	
 	SDL_PumpEvents();
 	result = SDL_PeepEvents(&event, 1, SDL_PEEKEVENT, mask);
+
+	if(noargs)
+		return PyEvent_New(&event);
 	return PyInt_FromLong(result == 1);
 }
 
@@ -748,7 +896,7 @@ static PyObject* event_post(PyObject* self, PyObject* args)
 	event.user.data2 = userobj;
 
 	if(SDL_PushEvent(&event) == -1)
-		return RAISE(PyExc_SDLError, SDL_GetError());
+		return RAISE(PyExc_SDLError, "Event queue full");
 
 	RETURN_NONE
 }
@@ -895,7 +1043,8 @@ static PyMethodDef event_builtins[] =
 	{ "pump", pump, 1, doc_pump },
 	{ "wait", pygame_wait, 1, doc_wait },
 	{ "poll", poll, 1, doc_poll },
-	{ "get", get, 1, doc_get },
+	{ "clear", event_clear, 1, doc_event_clear },
+	{ "get", event_get, 1, doc_event_get },
 	{ "peek", event_peek, 1, doc_peek },
 	{ "post", event_post, 1, doc_post },
 
