@@ -24,6 +24,7 @@
  *  font module for pygame
  */
 #define PYGAMEAPI_FONT_INTERNAL
+#include <stdio.h>
 #include <string.h>
 #include "pygame.h"
 #include "font.h"
@@ -35,7 +36,7 @@ static PyObject* PyFont_New(TTF_Font*);
 #define PyFont_Check(x) ((x)->ob_type == &PyFont_Type)
 
 static int font_initialized = 0;
-static char* font_defaultname = "bluebold.ttf";
+static char* font_defaultname = "freesansbold.ttf";
 static char* font_defaultpath = NULL;
 static PyObject* self_module = NULL;
 
@@ -105,7 +106,7 @@ static PyObject* font_autoinit(PyObject* self, PyObject* arg)
     /*DOC*/    "pygame.font.quit() -> none\n"
     /*DOC*/    "uninitialize the font module\n"
     /*DOC*/    "\n"
-    /*DOC*/    "Manually uninitialize SDL's video subsystem. It is safe to call\n"
+    /*DOC*/    "Manually uninitialize SDL_ttf's font system. It is safe to call\n"
     /*DOC*/    "this if font is currently not initialized.\n"
     /*DOC*/ ;
 
@@ -194,6 +195,7 @@ static PyObject* font_get_height(PyObject* self, PyObject* args)
     /*DOC*/    "\n"
     /*DOC*/    "Returns the descent for the font. The descent is the number of\n"
     /*DOC*/    "pixels from the font baseline to the bottom of the font.\n"
+    /*DOC*/    "With most fonts this is a negative number.\n"
     /*DOC*/ ;
 
 static PyObject* font_get_descent(PyObject* self, PyObject* args)
@@ -404,6 +406,9 @@ static PyObject* font_set_underline(PyObject* self, PyObject* args)
     /*DOC*/    "\n"
     /*DOC*/    "Also, rendering smooth text with underlines will crash with SDL_ttf\n"
     /*DOC*/    "less that version 2.0, be careful.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "If you pass an empty string, render() will return a blank surface\n"
+    /*DOC*/    "1 pixel wide and the same height as the font.\n"
     /*DOC*/ ;
 
 static PyObject* font_render(PyObject* self, PyObject* args)
@@ -426,20 +431,37 @@ static PyObject* font_render(PyObject* self, PyObject* args)
 		if(!RGBAFromObj(bg_rgba_obj, rgba))
 			return RAISE(PyExc_TypeError, "Invalid background RGBA argument");
 		backg.r = rgba[0]; backg.g = rgba[1]; backg.b = rgba[2];
-	}	
+	}
 
-	if(PyUnicode_Check(text))
+
+    	if(!PyObject_IsTrue(text))
 	{
-		Py_UNICODE* string = PyUnicode_AsUnicode(text);
+	    int height = TTF_FontHeight(font);
+	    surf = SDL_CreateRGBSurface(SDL_SWSURFACE, 1, height, 32, 0xff<<16, 0xff<<8, 0xff, 0);
+	    if(bg_rgba_obj)
+	    {
+	    	Uint32 c = SDL_MapRGB(surf->format, backg.r, backg.g, backg.b);
+	    	SDL_FillRect(surf, NULL, c);
+	    }
+	    else
+	    	SDL_SetColorKey(surf, SDL_SRCCOLORKEY, 0);
+	}
+	else if(PyUnicode_Check(text))
+	{
+		PyObject* strob = PyEval_CallMethod(text, "encode", "(s)", "utf-8");
+		char *string = PyString_AsString(strob);
+
 		if(aa)
 		{
 			if(!bg_rgba_obj)
-				surf = TTF_RenderUNICODE_Blended(font, string, foreg);
+				surf = TTF_RenderUTF8_Blended(font, string, foreg);
 			else
-				surf = TTF_RenderUNICODE_Shaded(font, string, foreg, backg);
+				surf = TTF_RenderUTF8_Shaded(font, string, foreg, backg);
 		}
 		else
-			surf = TTF_RenderUNICODE_Solid(font, string, foreg);
+			surf = TTF_RenderUTF8_Solid(font, string, foreg);
+
+		Py_DECREF(strob);
 	}
 	else if(PyString_Check(text))
 	{
@@ -458,10 +480,10 @@ static PyObject* font_render(PyObject* self, PyObject* args)
 		return RAISE(PyExc_TypeError, "text must be a string or unicode");
 
 	if(!surf)
-		return RAISE(PyExc_SDLError, SDL_GetError());
+		return RAISE(PyExc_SDLError, "SDL_ttf render failed");
 
 	if(!aa && bg_rgba_obj) /*turn off transparancy*/
-	{			
+	{
 		SDL_SetColorKey(surf, 0, 0);
 		surf->format->palette->colors[0].r = backg.r;
 		surf->format->palette->colors[0].g = backg.g;
@@ -496,8 +518,12 @@ static PyObject* font_size(PyObject* self, PyObject* args)
 
 	if(PyUnicode_Check(text))
 	{
-		Py_UNICODE* string = PyUnicode_AsUnicode(text);
-		TTF_SizeUNICODE(font, string, &w, &h);
+		PyObject* strob = PyEval_CallMethod(text, "encode", "(s)", "utf-8");
+		char *string = PyString_AsString(strob);
+
+		TTF_SizeUTF8(font, string, &w, &h);
+
+		Py_DECREF(strob);
 	}
 	else if(PyString_Check(text))
 	{
@@ -553,7 +579,7 @@ static PyObject* font_getattr(PyObject* self, char* attrname)
 		return Py_FindMethod(fontobj_builtins, self, attrname);
 
 	PyErr_SetString(PyExc_NameError, attrname);
-	return NULL; 
+	return NULL;
 }
 
 
@@ -569,14 +595,14 @@ static PyObject* font_getattr(PyObject* self, char* attrname)
     /*DOC*/    "normal python strings, as well as with unicode strings.\n"
     /*DOC*/ ;
 
-static PyTypeObject PyFont_Type = 
+static PyTypeObject PyFont_Type =
 {
 	PyObject_HEAD_INIT(NULL)
 	0,
 	"Font",
 	sizeof(PyFontObject),
 	0,
-	font_dealloc,	
+	font_dealloc,
 	0,
 	font_getattr,
 	0,
@@ -584,7 +610,7 @@ static PyTypeObject PyFont_Type =
 	0,
 	0,
 	NULL,
-	0, 
+	0,
 	(hashfunc)NULL,
 	(ternaryfunc)NULL,
 	(reprfunc)NULL,
@@ -594,17 +620,38 @@ static PyTypeObject PyFont_Type =
 
 
 
+/*font module methods*/
+
+    /*DOC*/ static char doc_get_default_font[] =
+    /*DOC*/    "pygame.font.get_default_font() -> string\n"
+    /*DOC*/    "get the name of the default font\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "returns the filename for the default truetype font.\n"
+    /*DOC*/ ;
+
+static PyObject* get_default_font(PyObject* self, PyObject* args)
+{
+	if(!PyArg_ParseTuple(args, ""))
+		return NULL;
+	return PyString_FromString(font_defaultname);
+}
+
+
+
 /*font module methods*/
 
     /*DOC*/ static char doc_Font[] =
     /*DOC*/    "pygame.font.Font(file, size) -> Font\n"
     /*DOC*/    "create a new font object\n"
     /*DOC*/    "\n"
-    /*DOC*/    "This will create a new font object. The given file must be a\n"
-    /*DOC*/    "filename to a TTF file. The font loader does not work with python\n"
-    /*DOC*/    "file-like objects. The size represents the height of the font in\n"
-    /*DOC*/    "pixels. The file argument can be 'None', which will use a plain\n"
-    /*DOC*/    "default font.\n"
+    /*DOC*/    "This will create a new font object. The given file can\n"
+    /*DOC*/    "be a filename or any python file-like object.\n"
+    /*DOC*/    "The size represents the height of the font in\n"
+    /*DOC*/    "pixels. The file argument can be 'None', which will\n"
+    /*DOC*/    "use a plain default font.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "You must have at least SDL_ttf-2.0.6 for file object\n"
+    /*DOC*/    "support. You can load TTF and FON fonts.\n"
     /*DOC*/ ;
 
 static PyObject* Font(PyObject* self, PyObject* args)
@@ -620,24 +667,55 @@ static PyObject* Font(PyObject* self, PyObject* args)
 	if(!font_initialized)
 		return RAISE(PyExc_SDLError, "font not initialized");
 
+	if(fontsize <= 1)
+		fontsize = 1;
+
 	if(fileobj == Py_None)
 	{
 		if(!font_defaultpath)
 			return RAISE(PyExc_RuntimeError, "default font not found");
-		filename = font_defaultpath;
+		/*keep sizing consistent with previous default fonts*/
+		fontsize = (int)(fontsize * .6875);
+                if(fontsize <= 1)
+                        fontsize = 1;
+
+                Py_BEGIN_ALLOW_THREADS
+                font = TTF_OpenFont(font_defaultpath, fontsize);
+                Py_END_ALLOW_THREADS
 	}
 	else if(PyString_Check(fileobj) || PyUnicode_Check(fileobj))
 	{
+		FILE* test;
+
 		if(!PyArg_ParseTuple(args, "si", &filename, &fontsize))
 			return NULL;
+
+                /*check if it is a valid file, else SDL_ttf segfaults*/
+                test = fopen(filename, "rb");
+                if(!test)
+                {
+                        return RAISE(PyExc_IOError, "unable to read font filename");
+                }
+                fclose(test);
+
+                Py_BEGIN_ALLOW_THREADS
+                font = TTF_OpenFont(filename, fontsize);
+                Py_END_ALLOW_THREADS
 	}
 	else
-		return RAISE(PyExc_TypeError, "font name must be string or None");
+        {
+#ifdef TTF_MAJOR_VERSION
+                SDL_RWops *rw;
+		if(!(rw = RWopsFromPython(fileobj)))
+			return NULL;
+                Py_BEGIN_ALLOW_THREADS
+                font = TTF_OpenFontIndexRW(rw, 1, fontsize, 0);
+                Py_END_ALLOW_THREADS
+#else
+                return RAISE(PyExc_NotImplementedError, "nonstring fonts require SDL_ttf-2.0.6");
+#endif
+        }
 
-	Py_BEGIN_ALLOW_THREADS
-	font = TTF_OpenFont(filename, fontsize);
-	Py_END_ALLOW_THREADS
-	
 	if(!font)
 		return RAISE(PyExc_RuntimeError, SDL_GetError());
 
@@ -655,9 +733,9 @@ static PyMethodDef font_builtins[] =
 	{ "init", font_init, 1, doc_init },
 	{ "quit", font_quit, 1, doc_quit },
 	{ "get_init", get_init, 1, doc_get_init },
+	{ "get_default_font", get_default_font, 1, doc_get_default_font },
 
 	{ "Font", Font, 1, doc_Font },
-
 	{ NULL, NULL }
 };
 
@@ -666,7 +744,7 @@ static PyMethodDef font_builtins[] =
 static PyObject* PyFont_New(TTF_Font* font)
 {
 	PyFontObject* fontobj;
-	
+
 	if(!font)
 		return RAISE(PyExc_RuntimeError, "unable to load font.");
 
@@ -689,6 +767,11 @@ static PyObject* PyFont_New(TTF_Font* font)
     /*DOC*/    "Font objects. The module by itself only has routines to\n"
     /*DOC*/    "initialize the module and create Font objects with\n"
     /*DOC*/    "pygame.font.Font().\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "You can load fonts from the standard system fonts by using the\n"
+    /*DOC*/    "pygame.font.SysFont() method. There are also other functions to\n"
+    /*DOC*/    "help you work with system fonts.\n"
+    /*DOC*/    "\n"
     /*DOC*/ ;
 
 PYGAME_EXPORT
