@@ -44,15 +44,69 @@ static int request_size = MIX_DEFAULT_FORMAT;
 static int request_stereo = MIX_DEFAULT_CHANNELS;
 static int request_chunksize = MIX_DEFAULT_CHUNKSIZE;
 
+struct ChannelData
+{
+    PyObject* sound;
+    PyObject* queue;
+    int endevent;
+};
+static struct ChannelData *channeldata = NULL;
+static int numchanneldata = 0;
+
 Mix_Music** current_music;
+Mix_Music** queue_music;
+
+
+static void endsound_callback(int channel)
+{
+    if(channeldata)
+    {
+	if(channeldata[channel].endevent && SDL_WasInit(SDL_INIT_VIDEO))
+	{
+	    SDL_Event e;
+	    memset(&e, 0, sizeof(e));
+	    e.type = channeldata[channel].endevent;
+	    SDL_PushEvent(&e);
+	}
+	if(channeldata[channel].queue)
+	{
+	    int channelnum;
+	    Mix_Chunk* sound = PySound_AsChunk(channeldata[channel].queue);
+	    Py_XDECREF(channeldata[channel].sound);
+	    channeldata[channel].sound = channeldata[channel].queue;
+	    channeldata[channel].queue = NULL;
+	    channelnum = Mix_PlayChannelTimed(channel, sound, 0, -1);
+	    if(channelnum != -1)
+	    	Mix_GroupChannel(channelnum, (int)sound);
+	}
+	else
+	{
+	    Py_XDECREF(channeldata[channel].sound);
+	    channeldata[channel].sound = NULL;
+	}
+    }
+}
 
 
 static void autoquit(void)
 {
+        int i;
 	if(SDL_WasInit(SDL_INIT_AUDIO))
 	{
 		Mix_HaltMusic();
 
+                if(channeldata)
+                {
+                    for(i=0; i<numchanneldata; ++i)
+		    {
+                        Py_XDECREF(channeldata[i].sound);
+			Py_XDECREF(channeldata[i].queue);
+		    }
+                    free(channeldata);
+                    channeldata = NULL;
+                    numchanneldata = 0;
+                }
+
 		if(current_music)
 		{
 			if(*current_music)
@@ -62,6 +116,15 @@ static void autoquit(void)
 			}
 			current_music = NULL;
 		}
+		if(queue_music)
+		{
+			if(*queue_music)
+			{
+				Mix_FreeMusic(*queue_music);
+				*queue_music = NULL;
+			}
+			queue_music = NULL;
+		}
 
 		Mix_CloseAudio();
 		SDL_QuitSubSystem(SDL_INIT_AUDIO);
@@ -80,11 +143,16 @@ static PyObject* autoinit(PyObject* self, PyObject* arg)
 
 	if(!PyArg_ParseTuple(arg, "|iiii", &freq, &size, &stereo, &chunk))
 		return NULL;
-	if(stereo)
+	if(stereo>=2)
 		stereo = 2;
 	else
 		stereo = 1;
 
+        if(size == 8) size = AUDIO_U8;
+        else if(size == -8) size = AUDIO_S8;
+        else if(size == 16) size = AUDIO_U16SYS;
+        else if(size == -16) size = AUDIO_S16SYS;
+        
 	/*make chunk a power of 2*/
 	for(i=0; 1<<i < chunk; ++i); //yes, semicolon on for loop
 	chunk = max(1<<i, 256);
@@ -92,6 +160,19 @@ static PyObject* autoinit(PyObject* self, PyObject* arg)
 	if(!SDL_WasInit(SDL_INIT_AUDIO))
 	{
 		PyGame_RegisterQuit(autoquit);
+                
+                if(!channeldata) /*should always be null*/
+                {
+                    numchanneldata = MIX_CHANNELS;
+                    channeldata = (struct ChannelData*)malloc(
+			    sizeof(struct ChannelData)*numchanneldata);
+                    for(i=0; i < numchanneldata; ++i)
+		    {
+                        channeldata[i].sound = NULL;
+			channeldata[i].queue = NULL;
+			channeldata[i].endevent = 0;
+		    }
+                }
 
 		if(SDL_InitSubSystem(SDL_INIT_AUDIO) == -1)
 			return PyInt_FromLong(0);
@@ -101,6 +182,11 @@ static PyObject* autoinit(PyObject* self, PyObject* arg)
 			SDL_QuitSubSystem(SDL_INIT_AUDIO);
 			return PyInt_FromLong(0);
 		}
+#if MIX_MAJOR_VERSION>=1 && MIX_MINOR_VERSION>=2 && MIX_PATCHLEVEL>=3
+                Mix_ChannelFinished(endsound_callback);
+#endif
+                
+              	Mix_VolumeMusic(127);
 	}
 	return PyInt_FromLong(1);
 }
@@ -249,6 +335,12 @@ static PyObject* snd_play(PyObject* self, PyObject* args)
 	if(channelnum == -1)
 		RETURN_NONE
 
+        Py_XDECREF(channeldata[channelnum].sound);
+	Py_XDECREF(channeldata[channelnum].queue);
+	channeldata[channelnum].queue = NULL;
+        channeldata[channelnum].sound = self;
+        Py_INCREF(self);
+                    
 	//make sure volume on this arbitrary channel is set to full
 	Mix_Volume(channelnum, 128);
 
@@ -370,6 +462,7 @@ static PyObject* snd_stop(PyObject* self, PyObject* args)
     /*DOC*/    "Set the play volume for this sound. This will effect any channels\n"
     /*DOC*/    "currently playing this sound, along with all subsequent calls to\n"
     /*DOC*/    "play. The value is 0.0 to 1.0.\n"
+
     /*DOC*/ ;
 
 static PyObject* snd_set_volume(PyObject* self, PyObject* args)
@@ -432,7 +525,7 @@ static PyMethodDef sound_builtins[] =
 
 static void sound_dealloc(PyObject* self)
 {
-	Mix_Chunk* chunk = PySound_AsChunk(self);
+    	Mix_Chunk* chunk = PySound_AsChunk(self);
 	Mix_FreeChunk(chunk);
 	PyObject_DEL(self);
 }
@@ -510,7 +603,61 @@ static PyObject* chan_play(PyObject* self, PyObject* args)
 	channelnum = Mix_PlayChannelTimed(channelnum, chunk, loops, playtime);
 	if(channelnum != -1)
 		Mix_GroupChannel(channelnum, (int)chunk);
+
+        Py_XDECREF(channeldata[channelnum].sound);
+	Py_XDECREF(channeldata[channelnum].queue);
+        channeldata[channelnum].sound = sound;
+	channeldata[channelnum].queue = NULL;
+        Py_INCREF(sound);
+
+        	
+	RETURN_NONE
+}
+
+
+
+    /*DOC*/ static char doc_chan_queue[] =
+    /*DOC*/    "Channel.queue(Sound) -> None\n"
+    /*DOC*/    "queue a sound on this channel\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "When you queue a sound on a channel, it will begin playing\n"
+    /*DOC*/    "immediately when the current playing sound finishes. Each\n"
+    /*DOC*/    "channel can only have a single Sound object queued. The\n"
+    /*DOC*/    "queued sound will only play when the current Sound finishes\n"
+    /*DOC*/    "naturally, not from another call to stop() or play().\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "If there is no currently playing sound on this Channel\n"
+    /*DOC*/    "it will begin playback immediately.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "This will only work with SDL_mixer greater than version 1.2.3\n"
+    /*DOC*/ ;
+
+static PyObject* chan_queue(PyObject* self, PyObject* args)
+{
+	int channelnum = PyChannel_AsInt(self);
+	PyObject* sound;
+	Mix_Chunk* chunk;
 	
+	if(!PyArg_ParseTuple(args, "O!", &PySound_Type, &sound))
+		return NULL;
+	chunk = PySound_AsChunk(sound);
+
+	if(!channeldata[channelnum].sound) /*nothing playing*/
+	{
+	    channelnum = Mix_PlayChannelTimed(channelnum, chunk, 0, -1);
+	    if(channelnum != -1)
+		    Mix_GroupChannel(channelnum, (int)chunk);
+
+            channeldata[channelnum].sound = sound;
+            Py_INCREF(sound);
+	}
+	else
+	{
+	    Py_XDECREF(channeldata[channelnum].queue);
+	    channeldata[channelnum].queue = sound;
+	    Py_INCREF(sound);
+	}
+        	
 	RETURN_NONE
 }
 
@@ -622,24 +769,38 @@ static PyObject* chan_unpause(PyObject* self, PyObject* args)
 
 
     /*DOC*/ static char doc_chan_set_volume[] =
-    /*DOC*/    "Channel.set_volume(val) -> None\n"
+    /*DOC*/    "Channel.set_volume(val, [stereoval]) -> None\n"
     /*DOC*/    "set volume for channel\n"
     /*DOC*/    "\n"
     /*DOC*/    "Sets the volume for the channel. The channel's volume level is\n"
     /*DOC*/    "mixed with the volume for the active sound object. The value is\n"
     /*DOC*/    "between 0.0 and 1.0.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "If mixer is using stereo, you can set the panning for audio\n"
+    /*DOC*/    "by supplying a volume for the left and right channels. If\n"
+    /*DOC*/    "SDL_mixer cannot set the panning, it will average the two\n"
+    /*DOC*/    "volumes. Panning requires SDL_mixer-1.2.1.\n"
     /*DOC*/ ;
 
 static PyObject* chan_set_volume(PyObject* self, PyObject* args)
 {
 	int channelnum = PyChannel_AsInt(self);
-	float volume;
+	float volume, stereovolume=-1.11f;
 
-	if(!PyArg_ParseTuple(args, "f", &volume))
+	if(!PyArg_ParseTuple(args, "f|f", &volume, &stereovolume))
 		return NULL;
 
 	MIXER_INIT_CHECK();
-
+#if MIX_MAJOR_VERSION>=1 && MIX_MINOR_VERSION>=2 && MIX_PATCHLEVEL>=1
+        if(stereovolume != -1.11f)
+            Mix_SetPanning(channelnum, (Uint8)(volume*255), (Uint8)(stereovolume*255));
+        else
+            Mix_SetPanning(channelnum, (Uint8)255, (Uint8)255);
+        volume = 1.0f;
+#else
+        if(stereovolume != -1.11f)
+            volume = (volume + stereovolume) * 0.5f;
+#endif
 	Mix_Volume(channelnum, (int)(volume*128));
 	RETURN_NONE
 }
@@ -670,10 +831,114 @@ static PyObject* chan_get_volume(PyObject* self, PyObject* args)
 
 
 
+    /*DOC*/ static char doc_chan_get_sound[] =
+    /*DOC*/    "Channel.get_sound() -> Sound\n"
+    /*DOC*/    "get the currently playing sound object\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Return the currently playing Sound object on this channel.\n"
+    /*DOC*/    "This will return None if there is nothing playing.\n"
+    /*DOC*/ ;
+
+static PyObject* chan_get_sound(PyObject* self, PyObject* args)
+{
+	int channelnum = PyChannel_AsInt(self);
+	PyObject* sound;
+
+	if(!PyArg_ParseTuple(args, ""))
+		return NULL;
+
+	sound = channeldata[channelnum].sound;
+	if(!sound)
+	    RETURN_NONE
+		    
+	Py_INCREF(sound);
+	return sound;
+}
+
+
+
+    /*DOC*/ static char doc_chan_get_queue[] =
+    /*DOC*/    "Channel.get_queue() -> Sound\n"
+    /*DOC*/    "get the currently queued sound object\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Return the currently queued Sound object on this channel.\n"
+    /*DOC*/    "This will return None if there is nothing queued.\n"
+    /*DOC*/ ;
+
+static PyObject* chan_get_queue(PyObject* self, PyObject* args)
+{
+	int channelnum = PyChannel_AsInt(self);
+	PyObject* sound;
+
+	if(!PyArg_ParseTuple(args, ""))
+		return NULL;
+
+	sound = channeldata[channelnum].queue;
+	if(!sound)
+	    RETURN_NONE
+		    
+	Py_INCREF(sound);
+	return sound;
+}
+
+
+
+
+
+    /*DOC*/ static char doc_chan_set_endevent[] =
+    /*DOC*/    "Channel.set_endevent([event_type]) -> None\n"
+    /*DOC*/    "set an endevent for a channel\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "When you set an endevent for a channel, that event type\n"
+    /*DOC*/    "will be put on the pygame event queue everytime a sound stops\n"
+    /*DOC*/    "playing on that channel. This is slightly different than the\n"
+    /*DOC*/    "music object end event, because this will trigger an event\n"
+    /*DOC*/    "anytime the music stops. If you call stop() or play() on the\n"
+    /*DOC*/    "channel, it will fire an event. An event will also be fired when\n"
+    /*DOC*/    "playback switches to a queued Sound.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Pass no argument to stop this channel from firing events\n"
+    /*DOC*/ ;
+
+static PyObject* chan_set_endevent(PyObject* self, PyObject* args)
+{
+	int channelnum = PyChannel_AsInt(self);
+	int event = SDL_NOEVENT;
+
+	if(!PyArg_ParseTuple(args, "|i", &event))
+		return NULL;
+
+	channeldata[channelnum].endevent = event;
+    	RETURN_NONE
+}
+
+
+    /*DOC*/ static char doc_chan_get_endevent[] =
+    /*DOC*/    "Channel.get_endevent() -> event_type\n"
+    /*DOC*/    "get the endevent for a channel\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns the end event type for this Channel. If the\n"
+    /*DOC*/    "return value is NOEVENT, then no events will be sent\n"
+    /*DOC*/    "when playback ends.\n"
+    /*DOC*/ ;
+
+static PyObject* chan_get_endevent(PyObject* self, PyObject* args)
+{
+	int channelnum = PyChannel_AsInt(self);
+
+	if(!PyArg_ParseTuple(args, ""))
+		return NULL;
+
+	return PyInt_FromLong(channeldata[channelnum].endevent);
+}
+
+
+
 
 static PyMethodDef channel_builtins[] =
 {
 	{ "play", chan_play, 1, doc_chan_play },
+	{ "queue", chan_queue, 1, doc_chan_queue },
 	{ "get_busy", chan_get_busy, 1, doc_chan_get_busy },
 	{ "fadeout", chan_fadeout, 1, doc_chan_fadeout },
 	{ "stop", chan_stop, 1, doc_chan_stop },
@@ -682,6 +947,12 @@ static PyMethodDef channel_builtins[] =
 	{ "set_volume", chan_set_volume, 1, doc_chan_set_volume },
 	{ "get_volume", chan_get_volume, 1, doc_chan_get_volume },
 
+	{ "get_sound", chan_get_sound, 1, doc_chan_get_sound },
+	{ "get_queue", chan_get_queue, 1, doc_chan_get_queue },
+		
+	{ "set_endevent", chan_set_endevent, 1, doc_chan_set_endevent },
+	{ "get_endevent", chan_get_endevent, 1, doc_chan_get_endevent },
+
 	{ NULL, NULL }
 };
 
@@ -774,12 +1045,26 @@ static PyObject* get_num_channels(PyObject* self, PyObject* args)
 
 static PyObject* set_num_channels(PyObject* self, PyObject* args)
 {
-	int numchans;
+	int numchans, i;
 	if(!PyArg_ParseTuple(args, "i", &numchans))
 		return NULL;
 
 	MIXER_INIT_CHECK();
 
+        if(numchans > numchanneldata)
+        {
+            channeldata= (struct ChannelData*)realloc(channeldata,
+		    sizeof(struct ChannelData*) * numchans);
+            for(i = numchanneldata; i < numchans; ++i)
+	    {
+		Py_XDECREF(channeldata[i].sound);
+		Py_XDECREF(channeldata[i].queue);
+		channeldata[i].sound = NULL;
+		channeldata[i].queue = NULL;
+	    }
+            numchanneldata = numchans;
+        }
+        
 	Mix_AllocateChannels(numchans);
 	RETURN_NONE
 }
@@ -860,7 +1145,7 @@ static PyObject* Channel(PyObject* self, PyObject* args)
     /*DOC*/    "channels, returns None.\n"
     /*DOC*/ ;
 
-static PyObject* find_channel(PyObject* self, PyObject* args)
+static PyObject* mixer_find_channel(PyObject* self, PyObject* args)
 {
 	int chan, force = 0;
 	if(!PyArg_ParseTuple(args, "|i", &force))
@@ -887,7 +1172,7 @@ static PyObject* find_channel(PyObject* self, PyObject* args)
     /*DOC*/    "milliseconds.\n"
     /*DOC*/ ;
 
-static PyObject* fadeout(PyObject* self, PyObject* args)
+static PyObject* mixer_fadeout(PyObject* self, PyObject* args)
 {
 	int time;
 	if(!PyArg_ParseTuple(args, "i", &time))
@@ -907,7 +1192,7 @@ static PyObject* fadeout(PyObject* self, PyObject* args)
     /*DOC*/    "Stop the playback on all mixer channels.\n"
     /*DOC*/ ;
 
-static PyObject* stop(PyObject* self, PyObject* args)
+static PyObject* mixer_stop(PyObject* self, PyObject* args)
 {
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
@@ -926,7 +1211,7 @@ static PyObject* stop(PyObject* self, PyObject* args)
     /*DOC*/    "Temporarily stops playback on all the mixer channels.\n"
     /*DOC*/ ;
 
-static PyObject* pause(PyObject* self, PyObject* args)
+static PyObject* mixer_pause(PyObject* self, PyObject* args)
 {
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
@@ -945,7 +1230,7 @@ static PyObject* pause(PyObject* self, PyObject* args)
     /*DOC*/    "Restarts playback of any paused channels.\n"
     /*DOC*/ ;
 
-static PyObject* unpause(PyObject* self, PyObject* args)
+static PyObject* mixer_unpause(PyObject* self, PyObject* args)
 {
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
@@ -1026,11 +1311,11 @@ static PyMethodDef mixer_builtins[] =
 
 	{ "get_busy", get_busy, 1, doc_get_busy },
 	{ "Channel", Channel, 1, doc_Channel },
-	{ "find_channel", find_channel, 1, doc_find_channel },
-	{ "fadeout", fadeout, 1, doc_fadeout },
-	{ "stop", stop, 1, doc_stop },
-	{ "pause", pause, 1, doc_pause },
-	{ "unpause", unpause, 1, doc_unpause },
+	{ "find_channel", mixer_find_channel, 1, doc_find_channel },
+	{ "fadeout", mixer_fadeout, 1, doc_fadeout },
+	{ "stop", mixer_stop, 1, doc_stop },
+	{ "pause", mixer_pause, 1, doc_pause },
+	{ "unpause", mixer_unpause, 1, doc_unpause },
 /*	{ "lookup_frequency", lookup_frequency, 1, doc_lookup_frequency },*/
 
 	{ "Sound", Sound, 1, doc_Sound },
@@ -1151,6 +1436,8 @@ void initmixer(void)
 		dict = PyModule_GetDict(music);
 		ptr = PyDict_GetItemString(dict, "_MUSIC_POINTER");
 		current_music = (Mix_Music**)PyCObject_AsVoidPtr(ptr);
+		ptr = PyDict_GetItemString(dict, "_QUEUE_POINTER");
+		queue_music = (Mix_Music**)PyCObject_AsVoidPtr(ptr);
 	}	
 	else /*music module not compiled? cleanly ignore*/
 	{
