@@ -30,18 +30,22 @@
 staticforward PyTypeObject PyRect_Type;
 #define PyRect_Check(x) ((x)->ob_type == &PyRect_Type)
 
+#if PYTHON_API_VERSION >= 1011 /*this is the python-2.2 constructor*/
+static PyObject* rect_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
+#endif
+
 
 
 GAME_Rect* GameRect_FromObject(PyObject* obj, GAME_Rect* temp)
 {
 	short val;
+	int length;
 
 	if(PyRect_Check(obj))
 		return &((PyRectObject*)obj)->r;
-
-	if(PySequence_Check(obj))
+	if(PySequence_Check(obj) && (length=PySequence_Length(obj))>0)
 	{
-		if(PySequence_Length(obj) == 4)
+		if(length == 4)
 		{
 			if(!ShortFromObjIndex(obj, 0, &val)) return NULL; temp->x = val;
 			if(!ShortFromObjIndex(obj, 1, &val)) return NULL; temp->y = val;
@@ -49,7 +53,7 @@ GAME_Rect* GameRect_FromObject(PyObject* obj, GAME_Rect* temp)
 			if(!ShortFromObjIndex(obj, 3, &val)) return NULL; temp->h = val;
 			return temp;
 		}
-		if(PySequence_Length(obj) == 2)
+		if(length == 2)
 		{
 			PyObject* sub = PySequence_GetItem(obj, 0);
 			if(!sub || !PySequence_Check(sub) || PySequence_Length(sub)!=2)
@@ -65,13 +69,30 @@ GAME_Rect* GameRect_FromObject(PyObject* obj, GAME_Rect* temp)
 			Py_DECREF(sub);
 			return temp;
 		}
-		if(PyTuple_Check(obj) && PyTuple_Size(obj) == 1) /*looks like an arg?*/
+		if(PyTuple_Check(obj) && length == 1) /*looks like an arg?*/
 		{
 			PyObject* sub = PyTuple_GET_ITEM(obj, 0);
 			if(sub)
 				return GameRect_FromObject(sub, temp);
 		}
 	}
+    if(PyObject_HasAttrString(obj, "rect"))
+    {
+		PyObject *rectattr;
+        GAME_Rect *returnrect;
+		rectattr = PyObject_GetAttrString(obj, "rect");
+        if(PyCallable_Check(rectattr)) /*call if it's a method*/
+        {
+            PyObject *rectresult = PyObject_CallObject(rectattr, NULL);
+            Py_DECREF(rectattr);
+            if(!rectresult)
+                return NULL;
+            rectattr = rectresult;
+        }
+        returnrect = GameRect_FromObject(rectattr, temp);
+        Py_DECREF(rectattr);
+        return returnrect;
+    }
 	return NULL;
 }
 
@@ -270,7 +291,7 @@ static PyObject* rect_inflate_ip(PyObject* oself, PyObject* args)
     /*DOC*/    "Rect.union(rectstyle) -> Rect\n"
     /*DOC*/    "makes new rectangle covering both inputs\n"
     /*DOC*/    "\n"
-    /*DOC*/    "Creates a new Rect to completely cover the\n"
+    /*DOC*/    "Returns a new Rect to completely cover the\n"
     /*DOC*/    "given input. There may be area inside the new\n"
     /*DOC*/    "Rect that is not covered by either input.\n"
     /*DOC*/ ;
@@ -348,7 +369,6 @@ static PyObject* rect_unionall(PyObject* oself, PyObject* args)
 	t = self->r.y;
 	r = self->r.x + self->r.w;
 	b = self->r.y + self->r.h;
-
 	size = PySequence_Length(list); /*warning, size could be -1 on error?*/
 	if(size < 1)
 		return PyRect_New4((short)l, (short)t, (short)(r-l), (short)(b-t));
@@ -362,13 +382,12 @@ static PyObject* rect_unionall(PyObject* oself, PyObject* args)
 			Py_XDECREF(obj);
 			break;
 		}
-		t = min(t, argrect->x);
-		l = min(l, argrect->y);
-		r = max(b, argrect->x+argrect->w);
+		l = min(l, argrect->x);
+		t = min(t, argrect->y);
+		r = max(r, argrect->x+argrect->w);
 		b = max(b, argrect->y+argrect->h);
 		Py_DECREF(obj);
 	}
-
 	return PyRect_New4((short)l, (short)t, (short)(r-l), (short)(b-t));
 }
 
@@ -377,7 +396,7 @@ static PyObject* rect_unionall(PyObject* oself, PyObject* args)
     /*DOC*/    "Rect.unionall_ip(sequence_of_rectstyles) -> None\n"
     /*DOC*/    "rectangle covering all inputs\n"
     /*DOC*/    "\n"
-    /*DOC*/    "Returns a new rectangle that completely covers all the\n"
+    /*DOC*/    "Resizes the rectangle to completely cover all the\n"
     /*DOC*/    "given inputs. There may be area inside the new\n"
     /*DOC*/    "rectangle that is not covered by the inputs.\n"
     /*DOC*/ ;
@@ -413,9 +432,9 @@ static PyObject* rect_unionall_ip(PyObject* oself, PyObject* args)
 			Py_XDECREF(obj);
 			break;
 		}
-		t = min(t, argrect->x);
-		l = min(l, argrect->y);
-		r = max(b, argrect->x+argrect->w);
+		l = min(l, argrect->x);
+		t = min(t, argrect->y);
+		r = max(r, argrect->x+argrect->w);
 		b = max(b, argrect->y+argrect->h);
 		Py_DECREF(obj);
 	}
@@ -524,8 +543,9 @@ static PyObject* rect_collidelist(PyObject* oself, PyObject* args)
 }
 
 
+
     /*DOC*/ static char doc_collidelistall[] =
-    /*DOC*/    "Rect.collidelistall(rectstyle list) -> int index\n"
+    /*DOC*/    "Rect.collidelistall(rectstyle list) -> index list\n"
     /*DOC*/    "find all overlapping rectangles\n"
     /*DOC*/    "\n"
     /*DOC*/    "Returns a list of the indexes that contain\n"
@@ -582,6 +602,112 @@ static PyObject* rect_collidelistall(PyObject* oself, PyObject* args)
 }
 
 
+    /*DOC*/ static char doc_collidedict[] =
+    /*DOC*/    "Rect.collidedict(dict if rectstyle keys) -> key/value pair\n"
+    /*DOC*/    "find overlapping rectangle in a dictionary\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns the key/value pair of the first rectangle key\n"
+    /*DOC*/    "in the dict that overlaps the base rectangle. Once an\n"
+    /*DOC*/    "overlap is found, this will stop checking the\n"
+    /*DOC*/    "remaining list. If no overlap is found, it will\n"
+    /*DOC*/    "return None.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Remember python dictionary keys must be immutable,\n"
+    /*DOC*/    "Rects are not immutable, so they cannot directly be,\n"
+    /*DOC*/    "dictionary keys. You can convert the Rect to a tuple\n"
+    /*DOC*/    "with the tuple() builtin command.\n"
+    /*DOC*/ ;
+
+static PyObject* rect_collidedict(PyObject* oself, PyObject* args)
+{
+	PyRectObject* self = (PyRectObject*)oself;
+	GAME_Rect *argrect, temp;
+	int loop=0;
+	PyObject* dict, *key, *val;
+	PyObject* ret = NULL;
+
+	if(!PyArg_ParseTuple(args, "O", &dict))
+		return NULL;
+	if(!PyDict_Check(dict))
+		return RAISE(PyExc_TypeError, "Argument must be a dict with rectstyle keys.");
+
+        while(PyDict_Next(dict, &loop, &key, &val))
+	{
+		if(!(argrect = GameRect_FromObject(key, &temp)))
+		{	
+			RAISE(PyExc_TypeError, "Argument must be a dict with rectstyle keys.");
+			break;
+		}
+		if(DoRectsIntersect(&self->r, argrect))
+		{
+			ret = Py_BuildValue("(OO)", key, val);
+			break;
+		}
+	}
+
+        if(!ret)
+        {
+            Py_INCREF(Py_None);
+            ret = Py_None;
+        }
+	return ret;	
+}
+
+
+    /*DOC*/ static char doc_collidedictall[] =
+    /*DOC*/    "Rect.collidedictall(rectstyle list) -> key/val list\n"
+    /*DOC*/    "find all overlapping rectangles\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns a list of the indexes that contain\n"
+    /*DOC*/    "rectangles overlapping the base rectangle. If no\n"
+    /*DOC*/    "overlap is found, it will return an empty\n"
+    /*DOC*/    "sequence.\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Remember python dictionary keys must be immutable,\n"
+    /*DOC*/    "Rects are not immutable, so they cannot directly be,\n"
+    /*DOC*/    "dictionary keys. You can convert the Rect to a tuple\n"
+    /*DOC*/    "with the tuple() builtin command.\n"
+    /*DOC*/ ;
+
+static PyObject* rect_collidedictall(PyObject* oself, PyObject* args)
+{
+	PyRectObject* self = (PyRectObject*)oself;
+	GAME_Rect *argrect, temp;
+	int loop=0;
+	PyObject* dict, *key, *val;
+	PyObject* ret = NULL;
+
+	if(!PyArg_ParseTuple(args, "O", &dict))
+		return NULL;
+	if(!PyDict_Check(dict))
+		return RAISE(PyExc_TypeError, "Argument must be a dict with rectstyle keys.");
+
+	ret = PyList_New(0);
+	if(!ret)
+		return NULL;
+
+        while(PyDict_Next(dict, &loop, &key, &val))
+	{
+		if(!(argrect = GameRect_FromObject(key, &temp)))
+		{	
+			Py_DECREF(ret);
+			return RAISE(PyExc_TypeError, "Argument must be a dict with rectstyle keys.");
+		}
+
+		if(DoRectsIntersect(&self->r, argrect))
+		{
+			PyObject* num = Py_BuildValue("(OO)", key, val);
+			if(!num)
+				return NULL;
+			PyList_Append(ret, num);
+			Py_DECREF(num);
+		}
+	}
+
+	return ret;	
+}
+
+
 
     /*DOC*/ static char doc_clip[] =
     /*DOC*/    "Rect.clip(rectstyle) -> Rect\n"
@@ -656,11 +782,16 @@ static PyObject* rect_contains(PyObject* oself, PyObject* args)
 		return RAISE(PyExc_TypeError, "Argument must be rect style object");
 
 	contained = (self->r.x <= argrect->x) && (self->r.y <= argrect->y) &&
+
 	            (self->r.x + self->r.w >= argrect->x + argrect->w) &&
+
 	            (self->r.y + self->r.h >= argrect->y + argrect->h) &&
+
 	            (self->r.x + self->r.w > argrect->x) &&
+
 	            (self->r.y + self->r.h > argrect->y);
 
+
 	return PyInt_FromLong(contained);
 }
 
@@ -747,6 +878,16 @@ static PyObject* rect_clamp_ip(PyObject* oself, PyObject* args)
 }
 
 
+/* for pickling */
+static PyObject* rect_reduce(PyObject* oself, PyObject* args)
+{
+	PyRectObject* self = (PyRectObject*)oself;
+        return Py_BuildValue("(O(iiii))", oself->ob_type,
+                    (int)self->r.x, (int)self->r.y, (int)self->r.w, (int)self->r.h);
+}
+
+
+
 static struct PyMethodDef rect_methods[] =
 {
 	{"normalize",		(PyCFunction)rect_normalize,	1, doc_normalize},
@@ -768,10 +909,15 @@ static struct PyMethodDef rect_methods[] =
 	{"colliderect", 	(PyCFunction)rect_colliderect,	1, doc_colliderect},
 	{"collidelist", 	(PyCFunction)rect_collidelist,	1, doc_collidelist},
 	{"collidelistall",	(PyCFunction)rect_collidelistall,1,doc_collidelistall},
+	{"collidedict", 	(PyCFunction)rect_collidedict,	1, doc_collidedict},
+	{"collidedictall",	(PyCFunction)rect_collidedictall,1,doc_collidedictall},
 	{"contains",		(PyCFunction)rect_contains,		1, doc_contains},
 /* these are totally unwritten. volunteers? */
 /*	{"cleanup",			(PyCFunction)rect_cleanup,		1, doc_cleanup}, */
 /*	{"remove",			(PyCFunction)rect_remove,		1, doc_remove}, */
+        
+        {"__reduce__",          (PyCFunction)rect_reduce, 0, NULL},
+        
 	{NULL,		NULL}
 };
 
@@ -1003,17 +1149,17 @@ static PyObject *rect_getattr(PyRectObject *self, char *name)
 	PyObject *ret = NULL;
 	GAME_Rect *r = &self->r;
 	
-	if(!strcmp(name, "top"))
+	if(!strcmp(name, "top") || !strcmp(name, "y"))
 		ret = PyInt_FromLong(r->y);
 	else if(!strcmp(name, "bottom"))
 		ret = PyInt_FromLong(r->y + r->h);
-	else if(!strcmp(name, "left"))
+	else if(!strcmp(name, "left") || !strcmp(name, "x"))
 		ret = PyInt_FromLong(r->x);
 	else if(!strcmp(name, "right"))
 		ret = PyInt_FromLong(r->x + r->w);
-	else if(!strcmp(name, "width"))
+	else if(!strcmp(name, "width") || !strcmp(name, "w"))
 		ret = PyInt_FromLong(r->w);
-	else if(!strcmp(name, "height"))
+	else if(!strcmp(name, "height") || !strcmp(name, "h"))
 		ret = PyInt_FromLong(r->h);
 	else if(!strcmp(name, "centerx"))
 		ret = PyInt_FromLong(r->x+r->w/2);
@@ -1041,8 +1187,10 @@ static PyObject *rect_getattr(PyRectObject *self, char *name)
 		ret = Py_BuildValue("(ii)", r->x + r->w / 2, r->y);
 	else if(!strcmp(name, "midbottom"))
 		ret = Py_BuildValue("(ii)", r->x + r->w / 2, r->y + r->h);
-	
-	else
+
+	else if(!strcmp(name, "__safe_for_unpickling__"))
+                ret = PyInt_FromLong(1);
+	else 
 		ret = Py_FindMethod(rect_methods, (PyObject *)self, name);
 
 	return ret;
@@ -1054,7 +1202,7 @@ static int rect_setattr(PyRectObject *self, char *name, PyObject *op)
 	short val1, val2;
 	GAME_Rect *r = &self->r;
 	
-	if(!strcmp(name, "top"))
+	if(!strcmp(name, "top") || !strcmp(name, "y"))
 	{
 		if(ShortFromObj(op, &val1))
 			ret = Rect_SetTop(r, val1);
@@ -1064,7 +1212,7 @@ static int rect_setattr(PyRectObject *self, char *name, PyObject *op)
 		if(ShortFromObj(op, &val1))
 			ret = Rect_SetBottom(r, val1);
 	}	
-	else if(!strcmp(name, "left"))
+	else if(!strcmp(name, "left") || !strcmp(name, "x"))
 	{
 		if(ShortFromObj(op, &val1))
 			ret = Rect_SetLeft(r, val1);
@@ -1074,12 +1222,12 @@ static int rect_setattr(PyRectObject *self, char *name, PyObject *op)
 		if(ShortFromObj(op, &val1))
 			ret = Rect_SetRight(r, val1);
 	}	
-	else if(!strcmp(name, "width"))
+	else if(!strcmp(name, "width") || !strcmp(name, "w"))
 	{
 		if(ShortFromObj(op, &val1))
 			ret = Rect_SetWidth(r, val1);
 	}	
-	else if(!strcmp(name, "height"))
+	else if(!strcmp(name, "height") || !strcmp(name, "h"))
 	{
 		if(ShortFromObj(op, &val1))
 			ret = Rect_SetHeight(r, val1);
@@ -1226,7 +1374,8 @@ static int rect_setattr(PyRectObject *self, char *name, PyObject *op)
     /*DOC*/    "height]]. Also, if a method takes a rectstyle\n"
     /*DOC*/    "argument as its only argument, you can simply pass\n"
     /*DOC*/    "four arguments representing xpos, ypos, width,\n"
-    /*DOC*/    "height.\n"
+    /*DOC*/    "height. A rectstyle argument can also be _any_ python\n"
+    /*DOC*/    "object with an attribute named 'rect'.\n"
     /*DOC*/ ;
 
 static PyTypeObject PyRect_Type = { 
@@ -1250,8 +1399,32 @@ static PyTypeObject PyRect_Type = {
 	(reprfunc)rect_str,			/*str*/
 
 	/* Space for future expansion */
-	0L,0L,0L,0L,
-	doc_Rect_MODULE /* Documentation string */
+	0L,0L,0L,
+#if PYTHON_API_VERSION >= 1011 /*PYTHON2.2*/
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_CHECKTYPES | Py_TPFLAGS_BASETYPE, /* tp_flags */
+#else
+	0,					/* tp_flags */
+#endif
+	doc_Rect_MODULE,    /* Documentation string */
+#if PYTHON_API_VERSION >= 1011 /*PYTHON2.2*/
+	0,					/* tp_traverse */
+	0,					/* tp_clear */
+	0,					/* tp_richcompare */
+	0,					/* tp_weaklistoffset */
+	0,					/* tp_iter */
+	0,					/* tp_iternext */
+	0,					/* tp_methods */
+	0,					/* tp_members */
+	0,					/* tp_getset */
+	0,					/* tp_base */
+	0,					/* tp_dict */
+	0,					/* tp_descr_get */
+	0,					/* tp_descr_set */
+	0,					/* tp_dictoffset */
+	0,					/* tp_init */
+	0,					/* tp_alloc */
+	rect_new,			/* tp_new */
+#endif
 };
 
 
@@ -1285,10 +1458,23 @@ static PyObject* RectInit(PyObject* self, PyObject* args)
 }
 
 
+#if PYTHON_API_VERSION >= 1011 /*this is the python-2.2 constructor*/
+static PyObject* rect_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+	GAME_Rect *argrect, temp;
+	if(!(argrect = GameRect_FromObject(args, &temp)))
+		return RAISE(PyExc_TypeError, "Argument must be rect style object");
+
+	return PyRect_New4(argrect->x, argrect->y, argrect->w, argrect->h);
+}
+#endif
+
 
 static PyMethodDef rect__builtins__[] =
 {
+#if PYTHON_API_VERSION < 1011 /*PYTHON2.2*/
 	{ "Rect", RectInit, 1, doc_Rect }, 
+#endif
 	{NULL, NULL}
 };
 
@@ -1304,8 +1490,6 @@ void initrect(void)
 	PyObject *module, *dict, *apiobj;
 	static void* c_api[PYGAMEAPI_RECT_NUMSLOTS];
 	
-	PyGAME_C_API[0] = PyGAME_C_API[0]; /*this cleans a compiler warning*/
-
 	/* Create the module and add the functions */
 	PyType_Init(PyRect_Type);
 
@@ -1314,6 +1498,9 @@ void initrect(void)
 	dict = PyModule_GetDict(module);
 
 	PyDict_SetItemString(dict, "RectType", (PyObject *)&PyRect_Type);
+#if PYTHON_API_VERSION >= 1011 /*this is the python-2.2 constructor*/
+	PyDict_SetItemString(dict, "Rect", (PyObject *)&PyRect_Type);
+#endif
 
 	/* export the c api */
 	c_api[0] = &PyRect_Type;
