@@ -30,6 +30,12 @@
 staticforward PyTypeObject PySurface_Type;
 static PyObject* PySurface_New(SDL_Surface* info);
 #define PySurface_Check(x) ((x)->ob_type == &PySurface_Type)
+extern int pygame_AlphaBlit(SDL_Surface *src, SDL_Rect *srcrect,
+                        SDL_Surface *dst, SDL_Rect *dstrect);
+
+#if PYTHON_API_VERSION >= 1011 /*this is the python-2.2 constructor*/
+static PyObject* surface_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
+#endif
 
 
 
@@ -161,7 +167,7 @@ static PyObject* surf_set_at(PyObject* self, PyObject* args)
 			byte_buf = (Uint8*)(pixels + y * surf->pitch) + x * 3;
 			*(byte_buf + (format->Rshift >> 3)) = rgba[0];
 			*(byte_buf + (format->Gshift >> 3)) = rgba[1];
-			*(byte_buf + (format->Bshift >> 3)) = rgba[2];	
+			*(byte_buf + (format->Bshift >> 3)) = rgba[2];
 			break;
 		default: /*case 4:*/
 			*((Uint32*)(pixels + y * surf->pitch) + x) = color;
@@ -215,7 +221,7 @@ static PyObject* surf_unmap_rgb(PyObject* self,PyObject* args)
 	SDL_Surface* surf = PySurface_AsSurface(self);
 	Uint32 col;
 	Uint8 r, g, b, a;
-	
+
 	if(!PyArg_ParseTuple(args, "i", &col))
 		return NULL;
 
@@ -321,7 +327,7 @@ static PyObject* surf_get_locked(PyObject* self, PyObject* args)
 
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
-	
+
 	return PyInt_FromLong(surf->lockcount);
 }
 
@@ -341,7 +347,7 @@ static PyObject* surf_get_palette(PyObject* self, PyObject* args)
 	SDL_Palette* pal = surf->format->palette;
 	PyObject* list;
 	int i;
-		
+
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
 
@@ -420,7 +426,7 @@ static PyObject* surf_set_palette(PyObject* self, PyObject* args)
 	PyObject* list, *item;
 	int i, len;
 	short r, g, b;
-	
+
 	if(!PyArg_ParseTuple(args, "O", &list))
 		return NULL;
 	if(!PySequence_Check(list))
@@ -437,7 +443,7 @@ static PyObject* surf_set_palette(PyObject* self, PyObject* args)
 	colors = (SDL_Color*)malloc(len * sizeof(SDL_Color));
 	if(!colors)
 		return NULL;
-	
+
 	for(i = 0; i < len; i++)
 	{
 		item = PySequence_GetItem(list, i);
@@ -488,7 +494,7 @@ static PyObject* surf_set_palette_at(PyObject* self, PyObject* args)
 		return NULL;
 	}
 
-	if(index >= pal->ncolors || index <= 0)
+	if(index >= pal->ncolors || index < 0)
 	{
 		PyErr_SetString(PyExc_IndexError, "index out of bounds");
 		return NULL;
@@ -541,10 +547,10 @@ static PyObject* surf_set_colorkey(PyObject* self, PyObject* args)
 	if(rgba_obj && rgba_obj!=Py_None)
 	{
 		if(PyNumber_Check(rgba_obj) && (intobj=PyNumber_Int(rgba_obj)))
-                {
+		{
 			color = (Uint32)PyInt_AsLong(intobj);
-                        Py_DECREF(intobj);
-                }
+			Py_DECREF(intobj);
+		}
 		else if(RGBAFromObj(rgba_obj, rgba))
 			color = SDL_MapRGBA(surf->format, rgba[0], rgba[1], rgba[2], rgba[3]);
 		else
@@ -560,7 +566,7 @@ static PyObject* surf_set_colorkey(PyObject* self, PyObject* args)
 
 	if(result == -1)
 		return RAISE(PyExc_SDLError, SDL_GetError());
-	
+
 	RETURN_NONE
 }
 
@@ -581,7 +587,7 @@ static PyObject* surf_get_colorkey(PyObject* self, PyObject* args)
 
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
-	
+
 	if(surf->flags & SDL_OPENGL)
 		return RAISE(PyExc_SDLError, "Cannot call on OPENGL Surfaces");
 
@@ -618,7 +624,7 @@ static PyObject* surf_set_alpha(PyObject* self, PyObject* args)
 	Uint32 flags = 0;
 	PyObject* alpha_obj = NULL, *intobj=NULL;
 	Uint8 alpha;
-	int result, alphaval=0, hasalpha=0;
+	int result, alphaval=255, hasalpha=0;
 
 	if(!PyArg_ParseTuple(args, "|Oi", &alpha_obj, &flags))
 		return NULL;
@@ -629,13 +635,13 @@ static PyObject* surf_set_alpha(PyObject* self, PyObject* args)
 	if(alpha_obj && alpha_obj!=Py_None)
 	{
 		if(PyNumber_Check(alpha_obj) && (intobj=PyNumber_Int(alpha_obj)))
-                {
-                        alphaval = (int)PyInt_AsLong(intobj);
-                        Py_DECREF(intobj);
-                }
-                else
-                        return RAISE(PyExc_TypeError, "invalid alpha argument");
-                hasalpha = 1;
+		{
+			alphaval = (int)PyInt_AsLong(intobj);
+			Py_DECREF(intobj);
+		}
+		else
+			return RAISE(PyExc_TypeError, "invalid alpha argument");
+		hasalpha = 1;
 	}
 	if(hasalpha)
 		flags |= SDL_SRCALPHA;
@@ -647,10 +653,10 @@ static PyObject* surf_set_alpha(PyObject* self, PyObject* args)
 	PySurface_Prep(self);
 	result = SDL_SetAlpha(surf, flags, alpha);
 	PySurface_Unprep(self);
-		
+
 	if(result == -1)
 		return RAISE(PyExc_SDLError, SDL_GetError());
-	
+
 	RETURN_NONE
 }
 
@@ -669,7 +675,7 @@ static PyObject* surf_get_alpha(PyObject* self, PyObject* args)
 
 	if(!PyArg_ParseTuple(args, ""))
 		return NULL;
-	
+
 	if(surf->flags & SDL_OPENGL)
 		return RAISE(PyExc_SDLError, "Cannot call on OPENGL Surfaces");
 
@@ -713,6 +719,11 @@ static PyObject* surf_convert(PyObject* self, PyObject* args)
 	if(!PyArg_ParseTuple(args, "|Oi", &argobject, &flags))
 		return NULL;
 
+        if(surf->flags & SDL_OPENGL)
+        {
+            return RAISE(PyExc_SDLError, "Cannot convert opengl display");
+        }
+
 	PySurface_Prep(self);
 	if(argobject)
 	{
@@ -722,7 +733,7 @@ static PyObject* surf_convert(PyObject* self, PyObject* args)
 			flags = src->flags | (surf->flags & (SDL_SRCCOLORKEY|SDL_SRCALPHA));
 			newsurf = SDL_ConvertSurface(surf, src->format, flags);
 		}
-		else 
+		else
 		{
 			short bpp;
 			SDL_PixelFormat format;
@@ -795,12 +806,12 @@ static PyObject* surf_convert(PyObject* self, PyObject* args)
 		}
 	}
 	else
-        {
-                if(SDL_WasInit(SDL_INIT_VIDEO))
-                        newsurf = SDL_DisplayFormat(surf);
-                else
-                        newsurf = SDL_ConvertSurface(surf, surf->format, surf->flags);
-        }
+	{
+		if(SDL_WasInit(SDL_INIT_VIDEO))
+			newsurf = SDL_DisplayFormat(surf);
+		else
+			newsurf = SDL_ConvertSurface(surf, surf->format, surf->flags);
+	}
 	PySurface_Unprep(self);
 
 	final = PySurface_New(newsurf);
@@ -832,7 +843,10 @@ static PyObject* surf_convert_alpha(PyObject* self, PyObject* args)
 	PyObject* final;
 	PySurfaceObject* srcsurf = NULL;
 	SDL_Surface* newsurf, *src;
-	
+
+	if(!SDL_WasInit(SDL_INIT_VIDEO))
+		return RAISE(PyExc_SDLError, "cannot convert without pygame.display initialized");
+
 	if(!PyArg_ParseTuple(args, "|O!", &PySurface_Type, &srcsurf))
 		return NULL;
 
@@ -869,21 +883,21 @@ static PyObject* surf_convert_alpha(PyObject* self, PyObject* args)
 static PyObject* surf_set_clip(PyObject* self, PyObject* args)
 {
 	SDL_Surface* surf = PySurface_AsSurface(self);
-        PyObject* item;
+	PyObject* item;
 	GAME_Rect *rect=NULL, temp;
 	int result;
 
 	if(PyTuple_Size(args))
 	{
-                item = PyTuple_GET_ITEM(args, 0);
-                if(!(item == Py_None && PyTuple_Size(args) == 1))
-                {
-                    rect = GameRect_FromObject(args, &temp);
-                    if(!rect)
-                            return RAISE(PyExc_ValueError, "invalid rectstyle object");
-                }
+		item = PyTuple_GET_ITEM(args, 0);
+		if(!(item == Py_None && PyTuple_Size(args) == 1))
+		{
+		    rect = GameRect_FromObject(args, &temp);
+		    if(!rect)
+			    return RAISE(PyExc_ValueError, "invalid rectstyle object");
+		}
 	}
-		
+
 	result = SDL_SetClipRect(surf, (SDL_Rect*)rect);
 	if(result == -1)
 		return RAISE(PyExc_SDLError, SDL_GetError());
@@ -933,7 +947,7 @@ static PyObject* surf_fill(PyObject* self, PyObject* args)
 	int result;
 	PyObject* rgba_obj;
 	Uint8 rgba[4];
-	
+
 	if(!PyArg_ParseTuple(args, "O|O", &rgba_obj, &r))
 		return NULL;
 
@@ -974,13 +988,104 @@ static PyObject* surf_fill(PyObject* self, PyObject* args)
 }
 
 
+/*this internal blit function is accessable through the C api*/
+int PySurface_Blit(PyObject *dstobj, PyObject *srcobj, SDL_Rect *dstrect, SDL_Rect *srcrect)
+{
+    SDL_Surface *src = PySurface_AsSurface(srcobj);
+    SDL_Surface *dst = PySurface_AsSurface(dstobj);
+    SDL_Surface *subsurface = NULL;
+    int result, suboffsetx=0, suboffsety=0;
+    SDL_Rect orig_clip, sub_clip;
+    int didconvert = 0;
+
+    /*passthrough blits to the real surface*/
+    if(((PySurfaceObject*)dstobj)->subsurface)
+    {
+	    PyObject *owner;
+	    struct SubSurface_Data *subdata;
+
+	    subdata = ((PySurfaceObject*)dstobj)->subsurface;
+	    owner = subdata->owner;
+            subsurface = PySurface_AsSurface(owner);
+	    suboffsetx = subdata->offsetx;
+	    suboffsety = subdata->offsety;
+
+	    while(((PySurfaceObject*)owner)->subsurface)
+	    {
+		subdata = ((PySurfaceObject*)owner)->subsurface;
+    		owner = subdata->owner;
+	        subsurface = PySurface_AsSurface(owner);
+	    	suboffsetx += subdata->offsetx;
+    	    	suboffsety += subdata->offsety;
+	    }
+
+	    SDL_GetClipRect(subsurface, &orig_clip);
+	    SDL_GetClipRect(dst, &sub_clip);
+	    sub_clip.x += suboffsetx;
+	    sub_clip.y += suboffsety;
+	    SDL_SetClipRect(subsurface, &sub_clip);
+	    dstrect->x += suboffsetx;
+	    dstrect->y += suboffsety;
+	    dst = subsurface;
+    }
+    else
+    {
+	    PySurface_Prep(dstobj);
+	    subsurface = NULL;
+    }
+
+    PySurface_Prep(srcobj);
+/*    Py_BEGIN_ALLOW_THREADS */
+
+    /*can't blit alpha to 8bit, crashes SDL*/
+    if(dst->format->BytesPerPixel==1 && (src->format->Amask || src->flags&SDL_SRCALPHA))
+    {
+	    didconvert = 1;
+	    src = SDL_DisplayFormat(src);
+    }
+
+    /*see if we should handle alpha ourselves*/
+    if(dst->format->Amask && (dst->flags&SDL_SRCALPHA) &&
+                !(src->format->Amask && !(src->flags&SDL_SRCALPHA)) && /*special case, SDL works*/
+                (dst->format->BytesPerPixel == 2 || dst->format->BytesPerPixel==4))
+    {
+        result = pygame_AlphaBlit(src, srcrect, dst, dstrect);
+    }
+    else
+    {
+        result = SDL_BlitSurface(src, srcrect, dst, dstrect);
+    }
+
+    if(didconvert)
+	    SDL_FreeSurface(src);
+
+/*    Py_END_ALLOW_THREADS */
+    if(subsurface)
+    {
+	    SDL_SetClipRect(subsurface, &orig_clip);
+	    dstrect->x -= suboffsetx;
+	    dstrect->y -= suboffsety;
+    }
+    else
+	PySurface_Unprep(dstobj);
+    PySurface_Unprep(srcobj);
+
+    if(result == -1)
+	    RAISE(PyExc_SDLError, SDL_GetError());
+    if(result == -2)
+	    RAISE(PyExc_SDLError, "Surface was lost");
+
+    return result != 0;
+}
+
+
 
 
     /*DOC*/ static char doc_surf_blit[] =
     /*DOC*/    "Surface.blit(source, destpos, [sourcerect]) -> Rect\n"
     /*DOC*/    "copy a one Surface to another.\n"
     /*DOC*/    "\n"
-    /*DOC*/    "The blitting will transfer one surface to another. It will\n"
+    /*DOC*/    "The blitting will copy pixels from the source. It will\n"
     /*DOC*/    "respect any special modes like colorkeying and alpha. If hardware\n"
     /*DOC*/    "support is available, it will be used. The given source is the\n"
     /*DOC*/    "Surface to copy from. The destoffset is a 2-number-sequence that\n"
@@ -999,7 +1104,7 @@ static PyObject* surf_fill(PyObject* self, PyObject* args)
     /*DOC*/    "rectangle sizes will be ignored.\n"
 #if 0  /* "" */
     /*DOC*/    "\n"
-    /*DOC*/    "Note that blitting surfaces with alpha onto 8bit destinations will\n"
+    /*DOC*/    "Blitting surfaces with pixel alphas onto an 8bit destination will\n"
     /*DOC*/    "not use the surface alpha values.\n"
 #endif /* "" */
     /*DOC*/ ;
@@ -1012,7 +1117,6 @@ static PyObject* surf_blit(PyObject* self, PyObject* args)
 	int dx, dy, result;
 	SDL_Rect dest_rect;
 	short sx, sy;
-	int didconvert = 0;
 
 	if(!PyArg_ParseTuple(args, "O!O|O", &PySurface_Type, &srcobject, &argpos, &argrect))
 		return NULL;
@@ -1021,9 +1125,6 @@ static PyObject* surf_blit(PyObject* self, PyObject* args)
 	if(dest->flags & SDL_OPENGL && !(dest->flags&(SDL_OPENGLBLIT&~SDL_OPENGL)))
 		return RAISE(PyExc_SDLError, "Cannot blit to OPENGL Surfaces (OPENGLBLIT is ok)");
 
-	if(dest->format->BytesPerPixel == 1 && (src->flags&SDL_SRCALPHA && src->format->Amask))
-		return RAISE(PyExc_SDLError, "Alpha blits to 8bit surfaces currently unimplemented");
-
 	if((src_rect = GameRect_FromObject(argpos, &temp)))
 	{
 		dx = src_rect->x;
@@ -1055,29 +1156,9 @@ static PyObject* surf_blit(PyObject* self, PyObject* args)
 	dest_rect.w = (unsigned short)src_rect->w;
 	dest_rect.h = (unsigned short)src_rect->h;
 
-	PySurface_Prep(self);
-	PySurface_Prep(srcobject);
-	Py_BEGIN_ALLOW_THREADS
-
-	/*can't blit alpha to 8bit, crashes SDL*/
-	if(dest->format->BytesPerPixel==1 && (src->format->Amask || src->flags&SDL_SRCALPHA))
-	{
-		didconvert = 1;
-		src = SDL_DisplayFormat(src);
-	}
-	result = SDL_BlitSurface(src, (SDL_Rect*)src_rect, dest, &dest_rect);
-	if(didconvert)
-		SDL_FreeSurface(src);
-
-	Py_END_ALLOW_THREADS
-	PySurface_Unprep(self);
-	PySurface_Unprep(srcobject);
-
-
-	if(result == -1)
-		return RAISE(PyExc_SDLError, SDL_GetError());
-	if(result == -2)
-		return RAISE(PyExc_SDLError, "Surface was lost");
+	result = PySurface_Blit(self, srcobject, &dest_rect, (SDL_Rect*)src_rect);
+	if(result != 0)
+	    return NULL;
 
 	return PyRect_New((GAME_Rect*)&dest_rect);
 }
@@ -1115,6 +1196,7 @@ static PyObject* surf_get_pitch(PyObject* self, PyObject* args)
 
 
 
+
     /*DOC*/ static char doc_surf_get_size[] =
     /*DOC*/    "Surface.get_size() -> x, y\n"
     /*DOC*/    "query the surface size\n"
@@ -1332,19 +1414,128 @@ static PyObject* surf_subsurface(PyObject* self, PyObject* args)
 		PyMem_Del(data);
 		return NULL;
 	}
-
 	Py_INCREF(self);
 	data->owner = self;
 	data->pixeloffset = pixeloffset;
+	data->offsetx = rect->x;
+	data->offsety = rect->y;
 	((PySurfaceObject*)subobj)->subsurface = data;
 
-
-
 	return subobj;
 }
 
 
 
+    /*DOC*/ static char doc_surf_get_offset[] =
+    /*DOC*/    "Surface.get_offset() -> x, y\n"
+    /*DOC*/    "get offset of subsurface\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns the X and Y position a subsurface is positioned\n"
+    /*DOC*/    "inside its parent. Will return 0,0 for surfaces that are\n"
+    /*DOC*/    "not a subsurface.\n"
+    /*DOC*/ ;
+
+static PyObject* surf_get_offset(PyObject* self, PyObject* args)
+{
+    	struct SubSurface_Data *subdata;
+    	subdata = ((PySurfaceObject*)self)->subsurface;
+	if(!subdata)
+    	    	return Py_BuildValue("(ii)", 0, 0);
+    	return Py_BuildValue("(ii)", subdata->offsetx, subdata->offsety);
+}
+
+
+    /*DOC*/ static char doc_surf_get_abs_offset[] =
+    /*DOC*/    "Surface.get_abs_offset() -> x, y\n"
+    /*DOC*/    "get absolute offset of subsurface\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns the absolute X and Y position a subsurface is positioned\n"
+    /*DOC*/    "inside its top level parent. Will return 0,0 for surfaces that are\n"
+    /*DOC*/    "not a subsurface.\n"
+    /*DOC*/ ;
+
+static PyObject* surf_get_abs_offset(PyObject* self, PyObject* args)
+{
+    	struct SubSurface_Data *subdata;
+	PyObject *owner;
+	int offsetx, offsety;
+
+    	subdata = ((PySurfaceObject*)self)->subsurface;
+	if(!subdata)
+    	    	return Py_BuildValue("(ii)", 0, 0);
+
+	subdata = ((PySurfaceObject*)self)->subsurface;
+	owner = subdata->owner;
+	offsetx = subdata->offsetx;
+	offsety = subdata->offsety;
+
+	while(((PySurfaceObject*)owner)->subsurface)
+	{
+	    subdata = ((PySurfaceObject*)owner)->subsurface;
+    	    owner = subdata->owner;
+	    offsetx += subdata->offsetx;
+    	    offsety += subdata->offsety;
+	}
+
+
+	return Py_BuildValue("(ii)", offsetx, offsety);
+}
+
+    /*DOC*/ static char doc_surf_get_parent[] =
+    /*DOC*/    "Surface.get_parent() -> Surface\n"
+    /*DOC*/    "get a subsurface parent\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns the Surface that is a parent of this subsurface.\n"
+    /*DOC*/    "Will return None if this is not a subsurface.\n"
+    /*DOC*/ ;
+
+static PyObject* surf_get_parent(PyObject* self, PyObject* args)
+{
+    	struct SubSurface_Data *subdata;
+    	subdata = ((PySurfaceObject*)self)->subsurface;
+	if(!subdata)
+    	    	RETURN_NONE
+
+    	Py_INCREF(subdata->owner);
+	return subdata->owner;
+}
+
+    /*DOC*/ static char doc_surf_get_abs_parent[] =
+    /*DOC*/    "Surface.get_abs_parent() -> Surface\n"
+    /*DOC*/    "get the toplevel surface for a subsurface\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Returns the top level Surface for this subsurface. If this is not\n"
+    /*DOC*/    "a subsurface it will return a reference to itself. You will always\n"
+    /*DOC*/    "get a valid surface from this method.\n"
+    /*DOC*/ ;
+static PyObject* surf_get_abs_parent(PyObject* self, PyObject* args)
+{
+    	struct SubSurface_Data *subdata;
+	PyObject *owner;
+
+    	subdata = ((PySurfaceObject*)self)->subsurface;
+	if(!subdata)
+	{
+	    Py_INCREF(self);
+	    return self;
+	}
+
+	subdata = ((PySurfaceObject*)self)->subsurface;
+	owner = subdata->owner;
+
+	while(((PySurfaceObject*)owner)->subsurface)
+	{
+	    subdata = ((PySurfaceObject*)owner)->subsurface;
+    	    owner = subdata->owner;
+	}
+
+	Py_INCREF(owner);
+	return owner;
+}
+
+
+
+
 static struct PyMethodDef surface_methods[] =
 {
 	{"get_at",			surf_get_at,		1, doc_surf_get_at },
@@ -1390,6 +1581,10 @@ static struct PyMethodDef surface_methods[] =
 	{"get_losses",		surf_get_losses,	1, doc_surf_get_losses },
 
 	{"subsurface",		surf_subsurface,	1, doc_surf_subsurface },
+	{"get_offset",		surf_get_offset,	1, doc_surf_get_offset },
+	{"get_abs_offset",	surf_get_abs_offset,	1, doc_surf_get_abs_offset },
+	{"get_parent",		surf_get_parent,	1, doc_surf_get_parent },
+	{"get_abs_parent",	surf_get_abs_parent,	1, doc_surf_get_abs_parent },
 
 	{NULL,		NULL}
 };
@@ -1402,9 +1597,13 @@ static void surface_dealloc(PyObject* self)
 {
 	PySurfaceObject* surf = (PySurfaceObject*)self;
 	struct SubSurface_Data* data = ((PySurfaceObject*)self)->subsurface;
+	int flags=0;
 
-	if(SDL_WasInit(SDL_INIT_VIDEO))
+	if(PySurface_AsSurface(surf))
+	    flags = PySurface_AsSurface(surf)->flags;
+	if(!(flags&SDL_HWSURFACE) || SDL_WasInit(SDL_INIT_VIDEO))
 	{
+	    	/*unsafe to free hardware surfaces without video init*/
 		while(surf->lockcount > 0)
 			PySurface_Unlock(self);
 		SDL_FreeSurface(surf->surf);
@@ -1415,7 +1614,7 @@ static void surface_dealloc(PyObject* self)
 		PyMem_Del(data);
 	}
 
-	PyObject_DEL(self);	
+	PyObject_DEL(self);
 }
 
 
@@ -1432,19 +1631,20 @@ PyObject* surface_str(PyObject* self)
 	SDL_Surface* surf = PySurface_AsSurface(self);
 	const char* type;
 
-        if(surf)
-        {
-            type = (surf->flags&SDL_HWSURFACE)?"HW":"SW";
-            sprintf(str, "<Surface(%dx%dx%d %s)>", surf->w, surf->h, surf->format->BitsPerPixel, type);
-        }
-        else
-        {
-            strcpy(str, "<Surface(Dead Display)>");
-        }
+	if(surf)
+	{
+	    type = (surf->flags&SDL_HWSURFACE)?"HW":"SW";
+	    sprintf(str, "<Surface(%dx%dx%d %s)>", surf->w, surf->h, surf->format->BitsPerPixel, type);
+	}
+	else
+	{
+	    strcpy(str, "<Surface(Dead Display)>");
+	}
 
 	return PyString_FromString(str);
 }
 
+#if PYTHON_API_VERSION < 1011 /*PYTHON2.2*/
     /*DOC*/ static char doc_Surface_MODULE[] =
     /*DOC*/    "Surface objects represent a simple memory buffer of pixels.\n"
     /*DOC*/    "Surface objects can reside in system memory, or in special\n"
@@ -1468,6 +1668,25 @@ PyObject* surface_str(PyObject* self)
     /*DOC*/    "module, which can treat the surfaces like large multidimensional\n"
     /*DOC*/    "arrays (and it's quite quick).\n"
     /*DOC*/ ;
+#endif
+    /*DOC*/ static char doc_Surface[] =
+    /*DOC*/    "pygame.Surface(size, [flags, [Surface|depth, [masks]]]) -> Surface\n"
+    /*DOC*/    "create a new Surface\n"
+    /*DOC*/    "\n"
+    /*DOC*/    "Creates a new surface object. Size is a 2-int-sequence containing\n"
+    /*DOC*/    "width and height. Depth is the number of bits used per pixel. If\n"
+    /*DOC*/    "omitted, depth will use the current display depth. Masks is a\n"
+    /*DOC*/    "four item sequence containing the bitmask for r,g,b, and a. If\n"
+    /*DOC*/    "omitted, masks will default to the usual values for the given\n"
+    /*DOC*/    "bitdepth. Flags is a mix of the following flags: SWSURFACE,\n"
+    /*DOC*/    "HWSURFACE, ASYNCBLIT, or SRCALPHA. (flags = 0 is the\n"
+    /*DOC*/    "same as SWSURFACE). Depth and masks can be substituted for\n"
+    /*DOC*/    "another surface object which will create the new surface with the\n"
+    /*DOC*/    "same format as the given one. When using default masks, alpha\n"
+    /*DOC*/    "will always be ignored unless you pass SRCALPHA as a flag.\n"
+    /*DOC*/    "For a plain software surface, 0 can be used for the flag. \n"
+    /*DOC*/    "A plain hardware surface can just use 1 for the flag.\n"
+    /*DOC*/ ;
 
 
 static PyTypeObject PySurface_Type =
@@ -1489,15 +1708,39 @@ static PyTypeObject PySurface_Type =
 	(hashfunc)NULL, 		/*hash*/
 	(ternaryfunc)NULL,		/*call*/
 	(reprfunc)NULL, 		/*str*/
-	0L,0L,0L,0L,
-	doc_Surface_MODULE /* Documentation string */
+	0L,0L,0L,
+#if PYTHON_API_VERSION >= 1011 /*PYTHON2.2*/
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_CHECKTYPES | Py_TPFLAGS_BASETYPE, /* tp_flags */
+	doc_Surface, /* Documentation string */
+#else
+	0,					/* tp_flags */
+	doc_Surface_MODULE, /* Documentation string */
+#endif
+#if PYTHON_API_VERSION >= 1011 /*PYTHON2.2*/
+	0,					/* tp_traverse */
+	0,					/* tp_clear */
+	0,					/* tp_richcompare */
+	0,					/* tp_weaklistoffset */
+	0,					/* tp_iter */
+	0,					/* tp_iternext */
+	0,					/* tp_methods */
+	0,					/* tp_members */
+	0,					/* tp_getset */
+	0,					/* tp_base */
+	0,					/* tp_dict */
+	0,					/* tp_descr_get */
+	0,					/* tp_descr_set */
+	0,					/* tp_dictoffset */
+	0,					/* tp_init */
+	0,					/* tp_alloc */
+	surface_new,		/* tp_new */
+#endif
 };
 
 
 static PyObject* PySurface_New(SDL_Surface* s)
 {
 	PySurfaceObject* surf;
-
 	if(!s) return RAISE(PyExc_SDLError, SDL_GetError());
 
 	surf = PyObject_NEW(PySurfaceObject, &PySurface_Type);
@@ -1515,25 +1758,6 @@ static PyObject* PySurface_New(SDL_Surface* s)
 
 /* surface module functions */
 
-    /*DOC*/ static char doc_Surface[] =
-    /*DOC*/    "pygame.Surface(size, [flags, [Surface|depth, [masks]]]) -> Surface\n"
-    /*DOC*/    "create a new Surface\n"
-    /*DOC*/    "\n"
-    /*DOC*/    "Creates a new surface object. Size is a 2-int-sequence containing\n"
-    /*DOC*/    "width and height. Depth is the number of bits used per pixel. If\n"
-    /*DOC*/    "omitted, depth will use the current display depth. Masks is a\n"
-    /*DOC*/    "four item sequence containing the bitmask for r,g,b, and a. If\n"
-    /*DOC*/    "omitted, masks will default to the usual values for the given\n"
-    /*DOC*/    "bitdepth. Flags is a mix of the following flags: SWSURFACE,\n"
-    /*DOC*/    "HWSURFACE, ASYNCBLIT, or SRCALPHA. (flags = 0 is the\n"
-    /*DOC*/    "same as SWSURFACE). Depth and masks can be substituted for\n"
-    /*DOC*/    "another surface object which will create the new surface with the\n"
-    /*DOC*/    "same format as the given one. When using default masks, alpha\n"
-    /*DOC*/    "will always be ignored unless you pass SRCALPHA as a flag.\n"
-    /*DOC*/    "For a plain software surface, 0 can be used for the flag. \n"
-    /*DOC*/    "A plain hardware surface can just use 1 for the flag.\n"
-    /*DOC*/ ;
-
 static PyObject* Surface(PyObject* self, PyObject* arg)
 {
 	Uint32 flags = 0;
@@ -1623,12 +1847,21 @@ static PyObject* Surface(PyObject* self, PyObject* arg)
 	return final;
 }
 
+#if PYTHON_API_VERSION >= 1011 /*this is the python-2.2 constructor*/
+static PyObject* surface_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+	return Surface(NULL, args);
+}
+#endif
+
 
 
 
 static PyMethodDef surface_builtins[] =
 {
+#if PYTHON_API_VERSION < 1011 /*PYTHON2.2*/
 	{ "Surface", Surface, 1, doc_Surface },
+#endif
 	{ NULL, NULL }
 };
 
@@ -1648,20 +1881,22 @@ void initsurface(void)
 
 	PyType_Init(PySurface_Type);
 
-
     /* create the module */
 	module = Py_InitModule3("surface", surface_builtins, doc_pygame_surface_MODULE);
 	dict = PyModule_GetDict(module);
 
 	PyDict_SetItemString(dict, "SurfaceType", (PyObject *)&PySurface_Type);
+#if PYTHON_API_VERSION >= 1011 /*this is the python-2.2 constructor*/
+	PyDict_SetItemString(dict, "Surface", (PyObject *)&PySurface_Type);
+#endif
 
 	/* export the c api */
 	c_api[0] = &PySurface_Type;
 	c_api[1] = PySurface_New;
+	c_api[2] = PySurface_Blit;
 	apiobj = PyCObject_FromVoidPtr(c_api, NULL);
 	PyDict_SetItemString(dict, PYGAMEAPI_LOCAL_ENTRY, apiobj);
 	Py_DECREF(apiobj);
-
 	/*imported needed apis*/
 	import_pygame_base();
 	import_pygame_rect();
@@ -1678,7 +1913,7 @@ void initsurface(void)
 			for(i = 0; i < PYGAMEAPI_SURFLOCK_NUMSLOTS; ++i)
 				PyGAME_C_API[i + PYGAMEAPI_SURFLOCK_FIRSTSLOT] = localptr[i];
 		}
+		Py_DECREF(lockmodule);
 	}
-	Py_XDECREF(lockmodule);
 }
 
